/**
 * \file gre/datatip.h
 * \brief GRE DataTip definitions
 *
 * \if NODOC
 * $Id: datatip.h_v 1.7 2012/07/23 22:45:07 bcolombe Exp $
 *
 * $Log: datatip.h_v $
 * Revision 1.7  2012/07/23 22:45:07  bcolombe
 * Add UnitDisplayed
 *
 * Revision 1.6  2012/06/12 21:44:11  mju
 * Add formatValue for numeric.
 *
 * Revision 1.5  2012/06/12 20:17:58  mju
 * Add thousands separator.
 *
 * Revision 1.4  2006/12/06 22:09:45  dwilliss
 * Added support for images mixed with text
 *
 * Revision 1.3  2006/10/02 20:52:02  mju
 * Add checkField.
 *
 * Revision 1.1  2006/09/28 13:04:23  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_DATATIP_H
#define	INC_GRE_DATATIP_H

#ifndef	INC_RVC_OBJECTNAME_H
	#include <rvc/objectname.h>
#endif

#ifndef	INC_MI32_ELEMTYPE_H
	#include <mi32/elemtype.h>
#endif


#ifndef	GENERATING_DOXYGEN_OUTPUT
class SERIALIZERITEM;
namespace RVC {
	class DBTABLE;
	}
namespace SPATMOD {
	namespace IMAGE {
		class STAGE_ARRAY;
		}
	}
#endif


namespace GRE {

//! DataTip parameters.
struct DATATIPPARMS {
	public:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class FORM;
		#endif

		//! Built-in datatip values.
		// At present only one value is allowed, however defined as bits so could allow multiple in future
		enum PREDEFINED {
			PREDEFINED_None =					0x00,
			PREDEFINED_ImageCellValue =	0x01,
			PREDEFINED_LinkTileName =		0x02,
			};

		RVC::OBJECTNAME TableName;					//!< Table name
		RVC::OBJECTNAME FieldName;					//!< Field name
		INT32 TableNum;								//!< Table number
		mutable INT32 FieldNum;						//!< Field number
		UINT8 Visible;									//!< Show this DataTip
		UINT8 IsNumeric;								//!< Set if field is numeric
		UINT8 UnitType;								//!< Field unit type, here for validation against DBFIELDINFO
		UINT8 m_Predefined;							//!< Predefined value to show
		UINT8 UnitShown;								//!< Unit to show field in
		UINT32 UnitDisplayed;							//!, Unit to display string in
		UINT8 DecPlaces;								//!< Decimal places for floating-point fields
		char m_Sep000;									//!< Separator character for thousands grouping in numbers
		UINT8 HighlightElem;							//!< Set to highlight element in view when datatip shown
		MISTRING Prefix;								//!< Label to place before value in string, defaults to table.field name
		MISTRING Suffix;								//!< Label to place after value in string, defaults to unit name

		//! Get serialization item definition array.
		DEPRECATED_MSG("Use SerialGetItemDef") static const SERIALIZERITEM* GetSerialItemDef();

		//! Get serialization item definition array.
		static const SERIALIZERITEM* SerialGetItemDef();

		//! Default constructor.
		DATATIPPARMS ();

		DATATIPPARMS (
			const DATATIPPARMS& rhs
			);

		DATATIPPARMS& operator= (
			const DATATIPPARMS& rhs
			);

		//! Check field name for validity in specified table
		ERRVALUE CheckField (
			RVC::DBTABLE& ObjTable
			);

		//! Do string formatting with prefix/suffix.
		void FormatString (
			MISTRING& ostr,						//!< String to append to
			const MISTRING& istr					//!< DataTip value
			) const;

		//! Do formatting of numeric value with prefix/suffix.
		void FormatValue (
			MISTRING& ostr,						//!< String to append to
			double value,							//!< DataTip value
			UINT8 unitnum = 0						//!< Value unit number
			) const;

		//! Get thousands separator.
		char GetThousandsSeparator () const
			{ return (m_Sep000); }

		//! Read string from table with minimal formatting
		//! @return TRUE if have value, FALSE if not, < 0 if error.
		int ReadString (
			RVC::DBTABLE& ObjTable,				//!< Table to read from
			ELEMTYPE ElemType,					//!< Element type
			INT32 ElemNum,							//!< Element number
			MISTRING& string,						//!< String to append result to
			SPATMOD::IMAGE::STAGE_ARRAY& images, //!< Images to insert into the string in place of {~IMG #} tags
			RVC::DBTABLE *pSourceTable = 0	//!< Source table if required
			) const;

		//! Read string from table and format with prefix/suffix.
		//! @return TRUE if have value, FALSE if not, < 0 if error.
		int ReadStringFormatted (
			RVC::DBTABLE& ObjTable,				//!< Table to read from
			ELEMTYPE ElemType,					//!< Element type
			INT32 ElemNum,							//!< Element number
			MISTRING& string,						//!< String to append result to
			SPATMOD::IMAGE::STAGE_ARRAY& images, //!< Images to insert into the string in place of {~IMG #} tags
			RVC::DBTABLE *pSourceTable = 0	//!< Source table if required
			) const;

		//! Set suffix to default based on unit type/shown.
		void SetDefaultSuffix ();

		//! Set thousands separator.
		void SetThousandsSeparator (
			char separator							//!< Separator, must be space ( ), comma (,), period (.) or zero for none
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MISTRING NumericToString (double value, UINT8 unitnum) const;
		#endif

	};

}	// End namespace GRE

#endif	// INC_GRE_DATATIP_H
