/**
 * \file mgui/ctrlangle.h
 * \brief Definitions for MGUI::CTRL_ANGLE
 *
 * \if NODOC
 * $Id: ctrlangle.h_v 1.1 2011/09/16 20:42:56 mju Exp $
 *
 * $Log: ctrlangle.h_v $
 * Revision 1.1  2011/09/16 20:42:56  mju
 * Initial revision
 *
 * \endif
 **/

#ifndef INC_MGUI_CTRLANGLE_H
#define	INC_MGUI_CTRLANGLE_H

#ifndef	INC_MGUI_CANVAS_H
#include <mgui/canvas.h>
#endif

namespace MGUI {

//===================================================================================================================
//! Control for displaying and setting "angle" value.
class CTRL_ANGLE : public MGUI::CANVAS {
	public:

		enum STYLE {
			STYLE_Default =					0x00,
			STYLE_NoUserChange =				0x01,		//!< User is not allowed to change value
			STYLE_AngleIncreasesCCW =		0x00,		//!< Angle increases in counterclockwise direction, ie right-hand cartesian (default)
			STYLE_AngleIncreasesCW =		0x02,		//!< Angle increases in clockwise direction, ie left-hand cartesian
			};
		
		//! Constructor.
		CTRL_ANGLE ();

		//! Destructor.
		virtual ~CTRL_ANGLE ();

		//! Create control with full angle range -180 to +180.
		void Create (
			LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			int DialRadius,						//!< Radius of "dial"
			STYLE style,							//!< Control style
			double origin = 0						//!< Direction in degrees (-180 to +180) in right-hand cartesian for 0-angle on display
			) { Create(ParentPane,DialRadius,-180,180,style,origin); }

		//! Create control with specific angle range.
		void Create (
			LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			int DialRadius,						//!< Radius of "dial"
			double MinAngle,						//!< Minimum angle in degrees (-360 to +360)
			double MaxAngle,						//!< Maximum angle in degrees, must be > MinAngle and <= MinAngle+360
			STYLE style,							//!< Control style
			double origin = 0						//!< Direction in degrees (-180 to +180) in right-hand cartesian for 0-angle on display
			);

		//! Create control with dual values and full angle range.
		void CreateDual (
			LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			int DialRadius,						//!< Radius of "dial"
			double MinAngle,						//!< Minimum angle (-360 to 0)
			double CenterValue2,					//!< Secondary value at center
			double EdgeValue2,					//!< Secondary value at edge (end of "needle")
			STYLE style,							//!< Control style
			double origin = 0						//!< Direction in degrees (-180 - +180) in right-hand cartesian for 0-angle on display
			);

		//! Get current angle value in degrees.
		double GetAngle () const;

		//! Get secondary value.
		double GetValue2 () const;

		//! Set delegate to call after value has changed.
		void SetDelegateOnChangeValue (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set delegate to call when user is actively changing value (by dragging control).
		void SetDelegateOnUserChanging (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set angle value.
		void SetAngle (
			double angle,				//!< Angle value
			bool notify = true		//!< Notify delegate of change
			);

		//! Set secondary value.
		void SetValue2 (
			double value,				//!< Value to set
			bool notify = true		//!< Notify delegate of change
			);

		//! Set both angle and secondary values.
		void SetValues (
			double angle,				//!< Angle to set
			double value2,				//!< Secondary value to set
			bool notify = true		//!< Notify delegate of change
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef CANVAS BASECLASS;

		class PRIV;
		PRIV *m_pPriv;

		virtual void v_OnPaint (MGD::CONTEXT*);
		virtual void v_OnPointerButtonDown (POINT, POINTERBUTTON button, KEYSTATE);
		virtual void v_OnPointerButtonUp (POINT, POINTERBUTTON button, KEYSTATE);
		virtual void v_OnPointerMove (POINT, POINTERBUTTON button, KEYSTATE, bool);

		CTRL_ANGLE (const CTRL_ANGLE&);
		CTRL_ANGLE& operator= (const CTRL_ANGLE&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================
}	// End namespace MGUI

#endif	// INC_MGUI_CTRLANGLE
