/**
 * \file mgui/gadget.h
 * \brief MGUI::GADGET class definitions.
 *
 * \if NODOC
 * $Id: gadget.h_v 1.7 2012/03/29 16:28:51 mju Exp $
 *
 * $Log: gadget.h_v $
 * Revision 1.7  2012/03/29 16:28:51  mju
 * Add canvasdevice_Stereo.
 *
 * Revision 1.6  2011/12/30 14:27:40  mju
 * Add canvasdevice_Pixmap and rename ...Buffer to ...Memory.
 *
 * Revision 1.5  2011/03/10 14:12:51  mju
 * Make onToolTipRequest public.
 *
 * Revision 1.4  2011/03/10 14:04:20  mju
 * Make toolcanvas::priv friend.
 *
 * Revision 1.3  2011/03/09 22:40:20  mju
 * Add tooltip support.
 *
 * Revision 1.2  2011/03/03 22:24:15  mju
 * Change to have dialog instead of form and other design changes.
 *
 * Revision 1.1  2011/02/25 18:59:31  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_GADGET_H
#define	INC_MGUI_GADGET_H

#ifndef	INC_MGUI_TOOLCANVAS_H
	#include <mgui/toolcanvas.h>
#endif


namespace MGUI {

//=====================================================================================================================

//! Base class for canvas gadgets.
//! A "gadget" is a simple graphic that is drawn on a canvas, into the canvas buffer or as an overlay or both.
//! The placement
class GADGET {
	public:

		//! Gadget placement.
		enum PLACEMENT {
			PLACEMENT_Center =			0x001,
			PLACEMENT_CenterLeft =		0x002,
			PLACEMENT_CenterRight =		0x004,
			PLACEMENT_BottomCenter =	0x008,
			PLACEMENT_BottomLeft =		0x010,
			PLACEMENT_BottomRight =		0x020,
			PLACEMENT_TopCenter =		0x040,
			PLACEMENT_TopLeft =			0x080,
			PLACEMENT_TopRight =			0x100,

			PLACEMENT_AnyLeft =			PLACEMENT_CenterLeft | PLACEMENT_BottomLeft | PLACEMENT_TopLeft,
			PLACEMENT_AnyRight =			PLACEMENT_CenterRight | PLACEMENT_BottomRight | PLACEMENT_TopRight,
			PLACEMENT_AnyBottom =		PLACEMENT_BottomCenter | PLACEMENT_BottomLeft | PLACEMENT_BottomRight,
			PLACEMENT_AnyTop =			PLACEMENT_TopCenter | PLACEMENT_TopLeft | PLACEMENT_TopRight,
			PLACEMENT_AnyCorner =		PLACEMENT_BottomLeft | PLACEMENT_BottomRight | PLACEMENT_TopLeft | PLACEMENT_TopRight,
			PLACEMENT_Any =				0x1FF
			};

		enum CANVASDEVICES {
			CANVASDEVICE_Memory =			0x01,		//!< Draw in canvas "memory" buffer
			CANVASDEVICE_Pixmap =			0x02,		//!< Draw in canvas "pixmap" buffer
			CANVASDEVICE_ScreenIfActive =	0x04,		//!< Draw on canvas "screen" if gadget is active
			CANVASDEVICE_ScreenAlways =	0x08,		//!< Draw on canvas "screen" always
			CANVASDEVICE_Stereo =			0x10,		//!< Draw in canvas "stereo" buffer
			};

		//! Flags used in construction.
		enum FLAGS {
			FLAG_None =					0,
			FLAG_AllowUserHide =		0x01,		//!< Allow user to hide gadget via default RMB popup menu
			FLAG_HasDlgSettings =	0x02,		//!< Has ability to create dialog for settings
			};

		//! Destructor
		virtual ~GADGET () = 0;

		//! Comparison operator for sorting.
		bool operator< (const GADGET& rhs) const
			{ return (m_priority < rhs.m_priority); }

		//! Determine if can draw gadget on specified canvas device type.
		bool CanDrawOnDeviceType (
			CANVAS::DEVICETYPE DeviceType
			) const;

		//! Show dialog to edit gadget settings.
		//! @return true if settings changed, false if not.
		bool DlgEditSettings (
			MDLGPARENT dlgparent
			);

		//! Draw gadget on canvas.
		void Draw (
			CANVAS::DEVICETYPE DeviceType
			);

		//! Get placement.
		PLACEMENT GetPlacement () const
			{ return (m_Placement); }

		//! Get position of gadget (top-left corner) in canvas coordinates.
		const MGUI::POINT& GetPosition () const
			{ return (m_pos); }

		//! Get priority.
		INT16 GetPriority () const
			{ return (m_priority); }

		//! Get size of gadget in pixels.
		const MGUI::POINT& GetSize () const
			{ return (m_size); }

		//! Determine if has position set.
		bool HasPosition () const
			{ return (m_hasposition); }

		//! Determine if gadget is visible.
		bool IsVisible () const
			{ return (m_visible); }

		//! \internal
		bool OnToolTipRequest (CTRL_TOOLTIP& tooltip, MGUI::POINT ptCanvas)
			{ return (v_OnToolTipRequest(tooltip,ptCanvas)); }

		//! Set delegate to call when placement changes.
		void SetDelegateOnChangePlacement (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnChangePlacement = delegate; }

		//! Set delegate to call when settings changed.
		void SetDelegateOnChangeSettings (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnChangeSettings = delegate; }

		//! Set delegate to call when visibility changes.
		void SetDelegateOnChangeVisible (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnChangeVisible = delegate; }

		//! Set placement.
		void SetPlacement (
			PLACEMENT Placement,			//!< Single placement value
			bool notify = true
			);

		//! Set which placements are allowed.
		void SetPlacementsAllowed (
			PLACEMENT PlacementsAllowed
			) { m_PlacementsAllowed = PlacementsAllowed; }

		//! Set priority for placement.
		void SetPriority (
			INT16 priority
			) { m_priority = priority; }

		//! Set whether gadget is visible.
		//! Does not update gadget placement or redraw canvas.  Gadgets are initially visible by default.
		//! Gadgets that are not visible will be ignored in placement and will receive no messages.
		void SetVisible (
			bool visible,
			bool notify = true
			);

	protected:

		//! Constructor
		explicit GADGET (
			TOOLCANVAS *pCanvas,
			PLACEMENT PlacementsAllowed,
			FLAGS flags = FLAG_None,
			CANVASDEVICES CanvasDevices = CANVASDEVICE_Memory
			);

		//! Draw gadget graphics.
		void Draw (MGD::CONTEXT *gc, CANVAS::DEVICETYPE DeviceType);

		//! Get "canvas" associated with gadget.
		TOOLCANVAS* GetCanvas () const
			{ return (m_pCanvas); }

		//! Determine if gadget has the pointer "captured".
		bool IsPointerCaptured () const;

		//! Set pointer cursor.
		void SetCursor (
			CURSORID cursorid
			) const { m_pCanvas->SetCursor(cursorid); }

		//! Translate canvas coordinates to gadget coordinates.
		MGUI::POINT TransCanvasToGadget (
			MGUI::POINT ptCanvas
			) const { return (MGUI::POINT(ptCanvas.m_x-m_pos.m_x,ptCanvas.m_y-m_pos.m_y)); }

		//! Translate gadget coordinates to canvas coordinates.
		MGUI::POINT TransGadgetToCanvas (
			MGUI::POINT ptGadget
			) const { return (MGUI::POINT(ptGadget.m_x+m_pos.m_x,ptGadget.m_y+m_pos.m_y)); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		TOOLCANVAS *m_pCanvas;
		FLAGS m_flags;
		CANVASDEVICES m_CanvasDevices;
		PLACEMENT m_PlacementsAllowed;
		PLACEMENT m_Placement;
		bool m_visible;
		bool m_hasposition;
		INT16 m_priority;
		MGUI::POINT m_pos;
		MGUI::POINT m_size;
		DELEGATE_VOID_NOPARMS m_DelegateOnChangePlacement;
		DELEGATE_VOID_NOPARMS m_DelegateOnChangeSettings;
		DELEGATE_VOID_NOPARMS m_DelegateOnChangeVisible;

		void ClearPosition ()
			{ m_hasposition = false; }
		void ComputeSize ()
			{ m_size = v_ComputeSize(); }
		bool ContainsPoint (MGUI::POINT ptCanvas) const
			{ return (m_hasposition && ptCanvas.m_x >= m_pos.m_x && ptCanvas.m_y >= m_pos.m_y && ptCanvas.m_x < m_pos.m_x + m_size.m_x && ptCanvas.m_y < m_pos.m_y + m_size.m_y); }
		void DetachCanvas ()
			{ m_pCanvas = 0; }
		bool OnKey (MGUI::POINT ptCanvas, MGUI::KEYCODE keycode, KEYSTATE keystate)
			{ return (v_OnKey(ptCanvas,keycode,keystate)); }
		bool OnPointerButtonDown (MGUI::POINT ptCanvas, POINTERBUTTON button, KEYSTATE keystate);
		bool OnPointerButtonUp (MGUI::POINT ptCanvas, POINTERBUTTON button, KEYSTATE keystate);
		void OnPointerEnter ()
			{ v_OnPointerEnter(); }
		void OnPointerLeave ()
			{ v_OnPointerLeave(); }
		void OnPointerMove (MGUI::POINT ptCanvas, POINTERBUTTON button, KEYSTATE keystate)
			{ v_OnPointerMove(ptCanvas,button,keystate); }
		void SetPosition (MGUI::POINT topleft)
			{ m_pos = topleft; m_hasposition = m_visible; }
		void SetPosition (INT16 x, INT16 y)
			{ m_pos.Set(x,y); m_hasposition = m_visible; }
		void ValidatePlacement ();

		#endif // GENERATING_DOXYGEN_OUTPUT

		// Overridables

		//! Compute gadget size.
		//! @return New gadget size.
		virtual MGUI::POINT v_ComputeSize () = 0;

		//! Called via DlgEditSettings.
		//! @return TRUE if settings changed, FALSE if not, < 0 if error.
		virtual int v_DlgEditSettings (MDLGPARENT dlgparent);

		//! Draw gadget graphic.
		//! Will not be called if canvas devices specified in constructor don't allow drawing on device type.
		virtual void v_Draw (MGD::CONTEXT *gc, CANVAS::DEVICETYPE DeviceType) = 0;

		//! Called when key pressed/released within gadget area.
		//! @return 'true' if key handled, 'false' if not.
		//! Default implementation does nothing.
		virtual bool v_OnKey (MGUI::POINT ptCanvas, MGUI::KEYCODE keycode, KEYSTATE keystate);

		//! Called when pointer button pressed within gadget area.
		//! @return true if button handled, false if not.
		//! Default implementation simply returns false.  If override then should also override v_OnPointerButtonUp
		//! and that should handle same button to avoid unwanted propagation.
		virtual bool v_OnPointerButtonDown (MGUI::POINT ptCanvas, POINTERBUTTON button, KEYSTATE keystate);

		//! Called when pointer button released.
		//! @return true if button handled, false if not.
		//! Default implementation simply returns false.
		virtual bool v_OnPointerButtonUp (MGUI::POINT ptCanvas, POINTERBUTTON button, KEYSTATE keystate);

		//! Called when pointer enters gadget location on canvas.
		//! Default implementation does nothing.
		virtual void v_OnPointerEnter ();

		//! Called when pointer leaves gadget location on canvas.
		//! Default implementation does nothing.
		virtual void v_OnPointerLeave ();

		//! Called when pointer is moved in gadget area or when captured.
		//! Default implementation does nothing.
		virtual void v_OnPointerMove (MGUI::POINT ptCanvas, POINTERBUTTON button, KEYSTATE keystate);

		//! Called when requesting tooltip.
		//! Default implementation sets text to press right button for settings and returns true if has capability.
		virtual bool v_OnToolTipRequest (CTRL_TOOLTIP& tooltip, MGUI::POINT& ptCanvas);

		friend class TOOLCANVAS;
		friend class TOOLCANVAS::PRIV;
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(GADGET::CANVASDEVICES)
DEFINE_ENUM_OP_BITWISE(GADGET::FLAGS)
DEFINE_ENUM_OP_BITWISE(GADGET::PLACEMENT)
#endif

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_GADGET_H
