/**
 * \file mi32/autoretry.h
 * \brief AUTORETRY class definitions.
 *
 * \if NODOC
 * $Id: autoretry.h_v 1.1 2012/08/29 15:53:29 mju Exp $
 *
 * $Log: autoretry.h_v $
 * Revision 1.1  2012/08/29 15:53:29  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_AUTORETRY_H
#define	INC_MI32_AUTORETRY_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

//! Base class for automatic retry of some action.
//! Typically used when an operation fails due to a temporary condition, such as a file being used by another process.
//! Allows control of retry interval, maximum time and maximum count, with potentially different settings in "batch"
//! and "interactive" processes.
class CLASSLIBEXPORT AUTORETRY {
	public:

		//! Construct with FILE_RAW reference.
		AUTORETRY ();

		//! Get retry interval in seconds.
		double GetRetryInterval () const
			{ return (m_RetryInterval); }

		//! Get maximum number of retries.
		UINT32 GetRetryMaxCount () const
			{ return (m_RetryMaxCount); }

		//! Get maximum retry time.
		double GetRetryMaxTime () const
			{ return (m_RetryMaxTime); }

		//! Set retry values for "batch" mode.
		//! Defaults for batch mode are: interval: 5 sec, time: 5 min, count: 50
		void SetRetryValuesBatch (
			double interval,					//!< Minimum interval between retry attempts
			double maxTime,					//!< Maximum time from first attempt
			UINT32 maxCount					//!< Maximum allowed number of retries
			);

		//! Set retry values for "interactive" mode.
		//! Default for interactive mode is to not retry.
		void SetRetryValuesInteractive (
			double interval,					//!< Minimum interval between retry attempts
			double maxTime,					//!< Maximum time from first attempt
			UINT32 maxCount					//!< Maximum allowed number of retries
			);

	protected:

		//! Destructor.
		~AUTORETRY ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		double m_RetryInterval;
		double m_RetryMaxTime;
		UINT32 m_RetryMaxCount;

		AUTORETRY (const AUTORETRY&);
		AUTORETRY& operator= (const AUTORETRY&);
		#endif
	};

#undef	CLASSLIBEXPORT
#endif	// INC_MI32_AUTORETRY_H
