/**
 * \file colorpal.h <mi32/colorpal.h>
 * \brief COLORPALETTE and COLORPALETTEMGR class definitions
 *
 * \if NODOC
 * $Id: colorpal.h_v 1.5 2013/06/05 13:42:47 mju Exp $
 *
 * $Log: colorpal.h_v $
 * Revision 1.5  2013/06/05 13:42:47  mju
 * Minor cleanup.
 *
 * Revision 1.4  2011/08/15 19:59:15  mju
 * Use color::model enum.
 *
 * Revision 1.3  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.2  2002/11/22 18:27:59  mju
 * Initial implementation finished and working.
 *
 * Revision 1.1  2002/11/19 22:58:30  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_COLORPAL_H
#define	INC_MI32_COLORPAL_H

#ifndef	INC_MI32_COLOR_H
#include <mi32/color.h>
#endif

#ifndef	INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif

#ifndef	INC_MI32_MILIST_H
#include <mi32/milist.h>
#endif

#ifndef	INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations
class FILEPATH;
class XMLNODE;
#endif

//=====================================================================================================================

//! Container for "extended" color.
//! Contains one or more models of a particular color, along with option name and other attributes.
//! Can be created from MicroImages "standard" XML \<color\> element.
class COLOREXT {
	public:

		//! Constructor.
		COLOREXT (
			) { }

		//! Add color to list.
		void AddColor (
			const COLOR& color
			);

		//! Create from XML "color" element node.
		ERRVALUE Create (
			const XMLNODE *xmlnode
			);

		//! Get color in specified COLOR::MODEL.
		//! If the color is not available in the specified model it will be returned in the first available model.
		const COLOR& GetColor (
			COLOR::MODEL model = COLOR::MODEL_COUNT	//!< Preferred color model, default will return first available
			) const;

		//! Get list of colors.
		const MILIST<COLOR>& GetColorList (
			) const { return (m_colors); }

		//! Get name.
		const MISTRING& GetName (
			) const { return (m_name); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		MISTRING m_name;
		MILIST<COLOR> m_colors;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//! Color palette container.
//! A color palette contains a set of items.  Each item can have a name as well as a list of
//! one or more representations of a particular color in various color models (COLOR::MODEL).
//! Color palettes are stored externally in an XML file.
class COLORPALETTE {
	public:

		//! Default constructor.
		COLORPALETTE ();

		//! Copy constructor.
		COLORPALETTE (const COLORPALETTE&);

		//! Destructor.
		~COLORPALETTE ();

		//! Assignment.
		COLORPALETTE& operator= (const COLORPALETTE&);

		//! Compute grid x,y from palette index based on layout.
		void ComputeGridFromIndex (
			int index,
			int& x,
			int& y
			) const;

		//! Compute palette index from grid x,y based on layout.
		int ComputeIndexFromGrid (
			int x,
			int y
			) const;

		//! Find matching color in palette.
		//! @return Index to first matching item, -1 if none found.
		int FindColor (
			const COLOR& color						//!< Color to search for
			) const;

		//! Get layout grid height in rows.
		int GetGridHeight (
			) const { return (m_GridHeight); }

		//! Get layout grid width in columns.
		int GetGridWidth (
			) const { return (m_GridWidth); }

		//! Get palette item.
		const COLOREXT& GetItem (
			int index									//!< Index to item
			) const;

		//! Get name of specified item.
		//! @return Name as MISTRING.
		const MISTRING& GetItemName (
			int index									//!< Index to item
			) const { return (GetItem(index).GetName()); }

		//! Get color of specified item.
		//! If the color is not available in the specified model it will be returned in the first available model.
		const COLOR& GetItemColor (
			int index,									//!< Index to item
			COLOR::MODEL model = COLOR::MODEL_COUNT	//!< Preferred color model, default will return first available
			) const { return (GetItem(index).GetColor(model)); }

		//! Get palette layout for user interface.
		const SIMPLE_ARRAY<WPOINT2D>& GetLayout (
			) const { return (m_Layout); }

		//! Get palette name.
		const MISTRING& GetName () const;

		//! Get number of palette items.
		int GetNumItems () const;

		//! Load palette from (XML) file.
		ERRVALUE Load (
			const FILEPATH& filepath
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		MISTRING m_Name;
		SIMPLE_ARRAY<WPOINT2D> m_Layout;
		COLOREXT *m_Items;
		int m_NumItems;
		int m_GridWidth;
		int m_GridHeight;

		void Clear ();
		void CreateMicroImagesDefault ();

		friend class COLORPALETTEMGR;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
typedef MILIST<COLORPALETTE> COLORPALETTELIST;

//=====================================================================================================================

//! Color palette list manager.
//! When first requested available palettes are read from the "palettes" subfolder under
//! the user's home directory, the executable directory, and MicroImages "shared" directory.
class COLORPALETTEMGR {
	public:

		//! Get list of available color palettes.
		static const COLORPALETTELIST& GetPaletteList ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		COLORPALETTELIST m_PaletteList;

		static const COLORPALETTEMGR& GetCommon ();

		COLORPALETTEMGR ();

		void AddPaletteFiles (const FILEPATH&);

		COLORPALETTEMGR (const COLORPALETTEMGR&);
		COLORPALETTEMGR& operator= (const COLORPALETTEMGR&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

#endif	// INC_MI32_COLORPAL_H
