/**
 * \file mi32/latlonfm.h 
 * \brief Definitions for Latitude/Longitude format conversions
 *
 * \if NODOC
 * $Id: latlonfm.h_v 1.27 2011/11/03 14:39:32 mju Exp $
 *
 * $Log: latlonfm.h_v $
 * Revision 1.27  2011/11/03 14:39:32  mju
 * Move strtodeg stuff to own hdr so can include only that from xdefns.
 *
 * Revision 1.26  2011/10/10 13:13:56  mju
 * Remove unneeded hdr.
 *
 * Revision 1.25  2011/09/01 20:17:40  mju
 * Exclude macros from dox.
 *
 * Revision 1.24  2008/03/18 22:03:22  scowan
 * Removed the precision mask from DecDeg definition.
 *
 * Revision 1.23  2008/01/31 15:14:20  mju
 * Add convertLatLonToString with precision based on confidence parm.
 *
 * Revision 1.22  2007/11/06 22:35:03  mju
 * Add flag_in_useSpaces.
 *
 * Revision 1.21  2007/10/15 13:38:24  mju
 * Add out_isLontitude.
 *
 * Revision 1.20  2007/09/20 20:29:14  mju
 * Add dft for getLatLonTypeStrAbbr.
 *
 * Revision 1.19  2007/09/20 20:20:06  mju
 * Move dmsconv defns to dmstype enum so can build own types.
 * Rename some types for clarity and implement other component arrangements.
 * Add getLatLonTypeStrAbbr.
 *
 * Revision 1.18  2007/02/26 17:23:33  mju
 * Add degtostrflag_AutoPrecision.
 *
 * Revision 1.17  2007/01/11 14:08:32  mju
 * Remove getLatLonFormatList.
 *
 * Revision 1.16  2007/01/05 22:26:53  dwilliss
 * Added GetLatLonTypeList
 *
 * Revision 1.15  2004/11/22 15:58:01  mju
 * Add more precise formats for DDD MM SS.ss... formats.
 *
 * Revision 1.14  2004/03/22 18:52:58  dwilliss
 * moving 3 functions into geom.dll
 *
 * Revision 1.13  2003/10/01 16:21:05  dwilliss
 * Fixed doxygen error
 *
 * Revision 1.12  2003/09/25 21:29:07  dwilliss
 * Don't capitalize enum any more. Genitor needed it, doxygen doesn't like it.
 *
 * Revision 1.11  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.10  2003/09/08 20:39:21  dwilliss
 * Added doxygen tags for start/end function groups
 *
 * Revision 1.9  2003/07/24 19:46:30  mju
 * Add conversion fn returning MISTRING to avoid buffer reuse issue.
 *
 * Revision 1.8  2003/06/03 16:42:12  scowan
 * Added listitemdef get standard conversion function.
 *
 * Revision 1.7  2002/09/09 13:52:06  mju
 * Remove deprecated #defines.
 *
 * Revision 1.6  2001/12/20 21:41:23  mju
 * Enum docs.
 *
 * Revision 1.5  2001/08/07 16:57:37  mju
 * Add FLAG_Longitude360.
 *
 * Revision 1.4  2000/05/25 13:50:20  mju
 * Add see-also comments.
 *
 * Revision 1.3  2000/05/25 13:14:08  mju
 * Need to include stddefns for MI_DLLIMPORT/EXPORT defns.
 *
 * Revision 1.2  2000/05/24 23:15:38  scowan
 * Moved some functions to dll.
 *
 * Revision 1.1  2000/05/11 15:51:54  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_LATLONFM_H
#define	INC_MI32_LATLONFM_H

#ifndef	INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif

#ifndef	INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif

#ifdef GEOMDLL
	#define LIBEXPORT MI_DLLEXPORT
#else
	#define LIBEXPORT MI_DLLIMPORT
#endif

/*----------------------------------------------------------------------------*/
/*		Lat/Lon definitions for ConverLatLonType()										*/
/*----------------------------------------------------------------------------*/

enum DMSTYPE {
	DMSTYPE_MASK_Components	=	0x000F,
	DMSTYPE_Deg =					0x0001,			//!< Decimal degrees
	DMSTYPE_DegMinSec =			0x0002,			//!< Degrees minutes seconds (default if none of other values set)
	DMSTYPE_DegMin =				0x0003,			//!< Degrees and minutes
	DMSTYPE_Sec =					0x0004,			//!< Decimal seconds
	DMSTYPE_MinSec =				0x0005,			//!< Minutes and seconds
	DMSTYPE_Min =					0x0006,			//!< Decimal minutes

	DMSTYPE_MASK_Hemisphere =	0x0030,
	DMSTYPE_DF =					0x0010,			//!< Hemisphere tag in front
	DMSTYPE_DB =					0x0020,			//!< Hemisphere tag in back

	DMSTYPE_CD =					0x0100,			//!< Input can be a double value for ConvertToString
	DMSTYPE_CW =					0x0200,			//!< Input can be a word array

	DMSTYPE_MASK_Precision =	0xF000,			//!< Precision mask
	DMSTYPE_0P =					0x0000,			//!< 0 decimal places
	DMSTYPE_1P =					0x1000,			//!< 1 decimal places
	DMSTYPE_2P =					0x2000,			//!< 2 decimal places
	DMSTYPE_3P =					0x3000,			//!< 3 decimal places
	DMSTYPE_4P =					0x4000,			//!< 4 decimal places
	DMSTYPE_5P =					0x5000,			//!< 5 decimal places
	DMSTYPE_6P =					0x6000,			//!< 6 decimal places
	DMSTYPE_7P =					0x7000,			//!< 7 decimal places
	DMSTYPE_8P =					0x8000,			//!< 8 decimal places
	DMSTYPE_9P =					0x9000,			//!< 9 decimal places

	DMSTYPE_None =					0,
	DMSTYPE_DecDeg =				DMSTYPE_Deg | DMSTYPE_CD,
	DMSTYPE_IDecDeg =				DMSTYPE_Deg | DMSTYPE_DF | DMSTYPE_CD,
	DMSTYPE_DDDdddddd =			DMSTYPE_Deg | DMSTYPE_CD | DMSTYPE_6P,
	DMSTYPE_IDDDdddddd =			DMSTYPE_Deg | DMSTYPE_DF | DMSTYPE_CD | DMSTYPE_6P,
	DMSTYPE_DDDMMSS =				DMSTYPE_DegMinSec | DMSTYPE_CD | DMSTYPE_CW,
	DMSTYPE_IDDDMMSS =			DMSTYPE_DegMinSec | DMSTYPE_DF | DMSTYPE_CW,
	DMSTYPE_DDDMMSSI =			DMSTYPE_DegMinSec | DMSTYPE_DB | DMSTYPE_CW,
	DMSTYPE_DDDMMSSS =			DMSTYPE_DegMinSec | DMSTYPE_CD | DMSTYPE_1P,
	DMSTYPE_IDDDMMSSS =			DMSTYPE_DegMinSec | DMSTYPE_DF | DMSTYPE_1P,
	DMSTYPE_DDDMMSSSI =			DMSTYPE_DegMinSec | DMSTYPE_DB | DMSTYPE_1P,
	DMSTYPE_DDDMMSSSS =			DMSTYPE_DegMinSec | DMSTYPE_CD | DMSTYPE_2P,
	DMSTYPE_IDDDMMSSSS =			DMSTYPE_DegMinSec | DMSTYPE_DF | DMSTYPE_2P,
	DMSTYPE_DDDMMSSSSI =			DMSTYPE_DegMinSec | DMSTYPE_DB | DMSTYPE_2P,
	DMSTYPE_DDDMMSSSSS =			DMSTYPE_DegMinSec | DMSTYPE_CD | DMSTYPE_3P,
	DMSTYPE_IDDDMMSSSSS =		DMSTYPE_DegMinSec | DMSTYPE_DF | DMSTYPE_3P,
	DMSTYPE_DDDMMSSSSSI =		DMSTYPE_DegMinSec | DMSTYPE_DB | DMSTYPE_3P,
	DMSTYPE_DDDMMSS_SS =			DMSTYPE_DegMinSec | DMSTYPE_2P | DMSTYPE_CD | DMSTYPE_CW,
	DMSTYPE_IDDDMMSS_SS =		DMSTYPE_DegMinSec | DMSTYPE_2P | DMSTYPE_DF | DMSTYPE_CW,
	DMSTYPE_DDDMMSS_SSI =		DMSTYPE_DegMinSec | DMSTYPE_2P | DMSTYPE_DB | DMSTYPE_CW,
	DMSTYPE_DDDMMSS_sss =		DMSTYPE_DegMinSec | DMSTYPE_3P | DMSTYPE_CD | DMSTYPE_CW,
	DMSTYPE_IDDDMMSS_sss =		DMSTYPE_DegMinSec | DMSTYPE_3P | DMSTYPE_DF | DMSTYPE_CW,
	DMSTYPE_DDDMMSS_sssI =		DMSTYPE_DegMinSec | DMSTYPE_3P | DMSTYPE_DB | DMSTYPE_CW,
	DMSTYPE_DDDMMSS_ssssss =	DMSTYPE_DegMinSec | DMSTYPE_6P | DMSTYPE_CD | DMSTYPE_CW,
	DMSTYPE_IDDDMMSS_ssssss =	DMSTYPE_DegMinSec | DMSTYPE_6P | DMSTYPE_DF | DMSTYPE_CW,
	DMSTYPE_DDDMMSS_ssssssI =	DMSTYPE_DegMinSec | DMSTYPE_6P | DMSTYPE_DB | DMSTYPE_CW,
	DMSTYPE_DDDMM_MM =			DMSTYPE_DegMinSec | DMSTYPE_DegMin | DMSTYPE_CD | DMSTYPE_CW,
	DMSTYPE_IDDDMM_MM =			DMSTYPE_DegMinSec | DMSTYPE_DegMin | DMSTYPE_DF | DMSTYPE_CW,
	DMSTYPE_DDDMM_MMI =			DMSTYPE_DegMinSec | DMSTYPE_DegMin | DMSTYPE_DB | DMSTYPE_CW,
	DMSTYPE_Seconds =				DMSTYPE_Sec | DMSTYPE_CD,
	DMSTYPE_ISeconds =			DMSTYPE_Sec | DMSTYPE_DF,
	DMSTYPE_SecondsI =			DMSTYPE_Sec | DMSTYPE_DB,
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DMSTYPE)
#endif

enum CONVDMSFLAGS {
	CONVDMSFLAG_None =						0,
	CONVDMSFLAG_IN_String =					0x00010000,		//!< Input buffer is a char* string
	CONVDMSFLAG_IN_WArray =					0x00020000,		//!< Input buffer is Unicode string
	CONVDMSFLAG_IN_Double =					0x00040000,		//!< Input buffer is a double
	CONVDMSFLAG_IN_UseSpaces =				0x00080000,		//!< Input string uses spaces as separator
	CONVDMSFLAG_OUT_String =				0x01000000,		//!< Output buffer is a char* string
	CONVDMSFLAG_OUT_WArray =				0x02000000,		//!< Output buffer is Unicode string
	CONVDMSFLAG_OUT_Double =				0x04000000,		//!< Output buffer is a double
	CONVDMSFLAG_OUT_PutSpaces =			0x10000000,		//!< Insert spaces between degrees, minutes, seconds if applicable
	CONVDMSFLAG_OUT_IsLatitude =			0x20000000,		//!< Use hemisphere indicator for Latitude, otherwise Longitude assumed
	CONVDMSFLAG_OUT_IsLongitude =			0x00000000,		//!< Use hemisphere indicator for Longitude (default)
	CONVDMSFLAG_OUT_LocalHemisphere =	0x40000000,		//!< Use localized hemisphere symbol
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(CONVDMSFLAGS)
#endif

struct DMSTYPEANDNAME {
	DMSTYPE type;
	TEXTID name;
	};

/*----------------------------------------------------------------------------*/
/*		Function Prototypes																		*/
/*----------------------------------------------------------------------------*/

//!\addtogroup latlonconv Latitude/Longitude Format Conversions
//!@{

//! Convert Latitude/Longitude value to string.
//!
//! @see ConvStringToLatLon
LIBEXPORT void ConvLatLonToString (
	double degrees,							//!< Value to convert
	char *string,								//!< String to hold result
	DMSTYPE desttype,							//!< Destination format
	CONVDMSFLAGS flags						//!< CONVDMS_OUT_...
	);

//! Convert Latitude/Longitude value to string with precision based on confidence value.
//! The "confidence" parameter should be based on the accuracy at which the value can be
//! determined.  For example, if converting from screen coordinates it would be based on the
//! difference (in degrees) between two neighboring pixel coordinates.  A value of zero
//! will be considered "unknown" and the precision will be determined from the "desttype" parameter.
//!
//! @see ConvStringToLatLon
LIBEXPORT void ConvLatLonToString (
	double degrees,							//!< Value to convert
	char *string,								//!< String to hold result
	DMSTYPE desttype,							//!< Destination format
	CONVDMSFLAGS flags,						//!< CONVDMS_OUT_...
	double confidence							//!< Confidence value in degrees for determining report precision
	);

//! Convert value to decimal degrees and/or another LatLon format.
//!
//! @return Converted value in degrees.
LIBEXPORT double ConvLatLonValue (
	const void *srcbuf,						//!< Pointer to source value, flags determine data type
	DMSTYPE srctype,							//!< Source format
	void *destbuf,								//!< Pointer to destination location, NULL if not needed
	DMSTYPE desttype,							//!< Destination format
	CONVDMSFLAGS flags						//!< Flags for input and output type
	);

//! Convert double-precision value to decimal degrees
//!
//! @return Converted value in degrees.
inline double ConvLatLonValue (
	double value,								//!< Value to convert
	DMSTYPE srctype							//!< Source format
	) {
	return (ConvLatLonValue(&value,srctype,0,DMSTYPE_None,CONVDMSFLAG_IN_Double));
	}

//! Convert string to decimal degrees.
//!
//! @return Converted value in degrees.
//!
//!	@see ConvertStringToDegs, ConvLatLonToString
LIBEXPORT double ConvStringToLatLon (
	const char *string,							//!< String to convert
	DMSTYPE srctype,								//!< String format
	CONVDMSFLAGS flags = CONVDMSFLAG_None	//!< None presently used
	);

#ifndef NO_DEPRECATED
//! Retrieve list of built-in latitude-longitude string formats.
//!
//! @return NULL-terminated array of strings containing format names.
//! \deprecated Use GetLatLonTypeList
LIBEXPORT const char** GetLatLonNames (void);

//! Get latitude-longitude format conversion type for given index.
//!
//! @see ConvLatLonValue, ConvLatLonToString, ConvStringToLatLon
//! \deprecated Use GetLatLonTypeList
LIBEXPORT DMSTYPE GetLatLonType (
	int index
	);
#endif

//! Retrieve list of built-in latitude-longitude string formats.
//! 
//! @see ConvLatLonValue, ConvLatLonToString, ConvStringToLatLon
LIBEXPORT ERRVALUE GetLatLonTypeList (
	SIMPLE_ARRAY<DMSTYPEANDNAME>& types
	);

//! Get string for showing format type.
//! @return Abbreviation string for use in user interface
LIBEXPORT MISTRING GetLatLonTypeStrAbbr (
	DMSTYPE type,
	CONVDMSFLAGS flags = CONVDMSFLAG_OUT_PutSpaces
	);

//!@}

#undef LIBEXPORT

#endif	// INC_MI32_LATLONFM_H
