/**
 * \file mi32/lidarlasrecordgrid.h
 * \brief LIDAR::LAS::RECORD_GRID definitions
 *
 * \if NODOC
 * $Id: lidarlasrecordgrid.h_v 1.2 2012/09/13 15:46:41 mju Exp $
 *
 * $Log: lidarlasrecordgrid.h_v $
 * Revision 1.2  2012/09/13 15:46:41  mju
 * Add readSet.
 *
 * Revision 1.1  2012/09/07 20:25:56  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_LIDARLASRECORDGRID_H
#define	INC_MI32_LIDARLASRECORDGRID_H

#ifndef	INC_MI32_LIDARLASRECORD_H
#include <mi32/lidarlasrecord.h>
#endif

#ifndef	INC_MI32_LIDARENUMS_H
#include <mi32/lidarenums.h>
#endif

#ifndef	INC_MI32_GRID2D_H
#include <mi32/grid2d.h>
#endif

#ifndef	INC_MI32_RANGE_H
#include <mi32/range.h>
#endif

#ifndef	INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif

#ifndef	INC_MI32_DOUBLEAR_H
#include <mi32/doublear.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
class RECORDBUFFER_LSBFIRST;
class UNSIGNEDSET;
namespace LIDAR {
	namespace LAS {
		class VLRLOC;
		}
	}
#endif


#ifdef RVCSYSDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif


namespace LIDAR {
namespace LAS {

//=====================================================================================================================

//! Base for "grid" records.
class CLASSLIBEXPORT RECORD_GRID : public RECORD_SCAN {
	public:

		//! Constructor.
		RECORD_GRID ();

		//! Destructor.
		virtual ~RECORD_GRID ();

		//! Calculate linear array index from grid X/Y indices.
		INT32 CalcIdx (
			INT32 GridX,
			INT32 GridY
			) const { return (m_GridSpec.CalcIdx(GridX,GridY)); }

		//! Calculate linear array index from POINTRECORD.
		//! @return -1 if out of range
		INT32 CalcIdxValidated (
			const POINTRECORD& pointrec
			) const;

		//! Create new grid.
		//! Must do before adding points.  Will call Initialize().
		ERRVALUE Create (
			LAS::FILE& file,
			const GRID2D& GridSpec			//!< Grid specification
			);

		//! Get grid specification.
		const GRID2D& GetGridSpec () const
			{ return (m_GridSpec); }

	protected:

		//! Load specification from buffer.
		ERRVALUE ReadSpec (
			LAS::FILE& file,
			const VLRLOC& vlrloc,
			UINT32 offset = 0
			);

		//! Save specification to buffer.
		void SaveSpec (
			RECORDBUFFER_LSBFIRST& buffer,
			UINT32 offset = 0
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		GRID2D m_GridSpec;

		RECORD_GRID (const RECORD_GRID&);
		RECORD_GRID& operator= (const RECORD_GRID&);
		#endif
	};

//=====================================================================================================================

//! Gridded point counts.
class CLASSLIBEXPORT RECORD_GRID_COUNT : public RECORD_GRID {
	public:

		//! Constructor.
		RECORD_GRID_COUNT ();

		//! Destructor.
		virtual ~RECORD_GRID_COUNT ();

		//! Get array of counts.
		const SIMPLE_ARRAY<UINT16>& GetArray () const
			{ return (m_Count); }

		//! Get count for specified grid cell.
		UINT16 GetCount (
			INT32 GridX,
			INT32 GridY
			) const { return (m_Count[CalcIdx(GridX,GridY)]); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		SIMPLE_ARRAY<UINT16> m_Count;

		// RECORD overrides.
		virtual int v_Read (LAS::FILE& file);
		virtual ERRVALUE v_Write (LAS::FILE& file, bool ForceExtFile) const;
		// RECORD_SCAN overrides.
		virtual ERRVALUE v_AddPoint (UINT64 PointIdx, const POINTRECORD& pointrec);
		virtual ERRVALUE v_Initialize (LAS::FILE& file);

		RECORD_GRID_COUNT (const RECORD_GRID_COUNT&);
		RECORD_GRID_COUNT& operator= (const RECORD_GRID_COUNT&);
		#endif
	};

//=====================================================================================================================

//! Gridded count of points by RETURNTYPE.
class CLASSLIBEXPORT RECORD_GRID_RETURNTYPE : public RECORD_GRID {
	public:

		//! Constructor.
		RECORD_GRID_RETURNTYPE ();

		//! Destructor.
		virtual ~RECORD_GRID_RETURNTYPE ();

		//! Get count for specified grid cell.
		UINT16 GetCount (
			INT32 GridX,
			INT32 GridY,
			POINTRETURNTYPE ReturnType
			) const { return (m_CountByReturnType[CalcIdx(GridX,GridY)][ReturnType]); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef UINT16 COUNTS[POINTRETURNTYPE_NUMTYPES];

		SIMPLE_ARRAY<COUNTS> m_CountByReturnType;

		// RECORD overrides.
		virtual int v_Read (LAS::FILE& file);
		virtual ERRVALUE v_Write (LAS::FILE& file, bool ForceExtFile) const;
		// RECORD_SCAN overrides.
		virtual ERRVALUE v_AddPoint (UINT64 PointIdx, const POINTRECORD& pointrec);
		virtual ERRVALUE v_Initialize (LAS::FILE& file);

		RECORD_GRID_RETURNTYPE (const RECORD_GRID_RETURNTYPE&);
		RECORD_GRID_RETURNTYPE& operator= (const RECORD_GRID_RETURNTYPE&);
		#endif
	};

//=====================================================================================================================

//! Gridded Z range for points.
class CLASSLIBEXPORT RECORD_GRID_RANGEZ : public RECORD_GRID {
	public:

		//! Constructor.
		RECORD_GRID_RANGEZ ();

		//! Destructor.
		virtual ~RECORD_GRID_RANGEZ ();

		//! Get array of counts.
		const DOUBLE_ARRAY<DOUBLE_RANGE>& GetArray () const
			{ return (m_Range); }

		//! Get Z range for specified grid cell.
		const DOUBLE_RANGE& GetRange (
			INT32 GridX,
			INT32 GridY
			) const { return (m_Range[CalcIdx(GridX,GridY)]); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		DOUBLE_ARRAY<DOUBLE_RANGE> m_Range;

		// RECORD overrides.
		virtual int v_Read (LAS::FILE& file);
		virtual ERRVALUE v_Write (LAS::FILE& file, bool ForceExtFile) const;
		// RECORD_SCAN overrides.
		virtual ERRVALUE v_AddPoint (UINT64 PointIdx, const POINTRECORD& pointrec);
		virtual ERRVALUE v_Initialize (LAS::FILE& file);

		RECORD_GRID_RANGEZ (const RECORD_GRID_RANGEZ&);
		RECORD_GRID_RANGEZ& operator= (const RECORD_GRID_RANGEZ&);
		#endif
	};

//=====================================================================================================================

class CLASSLIBEXPORT RECORD_GRID_INDEX : public RECORD_GRID {
	public:

		//! Constructor.
		RECORD_GRID_INDEX ();

		//! Destructor.
		virtual ~RECORD_GRID_INDEX ();

		//! Get UNSIGNEDSET pointer for specified grid cell.
		//! @return Pointer to UNSIGNEDSET, NULL if cell has no points.
		const UNSIGNEDSET* GetSetPtr (
			INT32 GridX,
			INT32 GridY
			) const { return (m_SetPtrs[CalcIdx(GridX,GridY)]); }

		//! Open index for specified file.
		//! @return TRUE if index exists, FALSE if not, < 0 if error.
		int Open (
			LAS::FILE& file
			);

		//! Read set for specified grid cell.
		//! @return TRUE if has points, FALSE if not, < 0 if error
		//! Previously read sets are kept in memory as long as the class instance exists.
		int ReadCell (
			LAS::FILE& file,
			INT32 GridX,
			INT32 GridY
			);

		//! Read set for specified rectangle.
		//! @return 1 if set contains all points in file, 0 if not, < 0 if error
		int ReadSet (
			LAS::FILE& file,
			const DRECT2D& srcrect,			//!< Rectangle in LAS point coordinates
			UNSIGNEDSET& setret				//!< Set returned
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		SIMPLE_ARRAY<UNSIGNEDSET*> m_SetPtrs;
		SIMPLE_ARRAY<UINT32> m_ItemSizes;
		SIMPLE_ARRAY<UINT32> m_ItemOffsets;
		VLRLOC *m_pVlrLoc;

		void Free ();

		// RECORD overrides.
		virtual int v_Read (LAS::FILE& file);
		virtual ERRVALUE v_Write (LAS::FILE& file, bool ForceExtFile) const;
		// RECORD_SCAN overrides.
		virtual ERRVALUE v_AddPoint (UINT64 PointIdx, const POINTRECORD& pointrec);
		virtual ERRVALUE v_Finalize ();
		virtual ERRVALUE v_Initialize (LAS::FILE& file);

		RECORD_GRID_INDEX (const RECORD_GRID_INDEX&);
		RECORD_GRID_INDEX& operator= (const RECORD_GRID_INDEX&);
		#endif
	};

//=====================================================================================================================

}	// End namespace LAS
}	// End namespace LIDAR

#undef	CLASSLIBEXPORT

#endif	// INC_MI32_LIDARLASRECORDGRID_H
