/******************************************************************************
 * \file mi32/mg2x.h
 * \brief MGD::DEVICE_X definitions.
 *
 * \if NODOC
 * $Log: mg2x.h_v $
 * Revision 1.35  2012/03/15 17:36:51  mju
 * Remove v_isX11.
 *
 * Revision 1.34  2012/03/05 21:19:26  mju
 * Fix unneeded scoping in override declaration.
 *
 * Revision 1.33  2011/12/15 21:41:26  mju
 * Don't inline virtual methods.
 *
 * Revision 1.32  2011/12/15 21:31:30  mju
 * Add stipple support.
 *
 * Revision 1.31  2011/05/26 19:16:06  mju
 * Add getDrawable.
 *
 * Revision 1.30  2010/01/20 18:44:50  mju
 * Add getDisplay.
 *
 * Revision 1.29  2009/09/01 21:26:36  mju
 * Add context::v_copyRect.
 *
 * Revision 1.27  2006/10/18 14:03:13  mju
 * Use miconstruct* instead of micon typedef.
 *
 * Revision 1.25  2005/03/07 16:54:56  dwilliss
 * Added virtual method for interface text metrics
 *
 * Revision 1.24  2004/05/25 17:44:14  dwilliss
 * Added INTERFACETEXTSTYLE enum and parameters to InterfaceText functions
 *
 * Revision 1.12  2002/11/13 23:36:24  dwilliss
 * Made some methods non-virtual.
 * Complex clip
 *
 * Revision 1.11  2002/10/03 22:06:32  dwilliss
 * Added InterfaceTextGetWidth(),  Gave InterfaceTextDrawString() an optional
 * parameter for visual order drawing
 *
 * Revision 1.10  2002/10/03 20:41:59  dwilliss
 * Added a ChangeWidget which is more efficient than ChangeDrawable
 *
 * Revision 1.1  2001/07/11 17:34:00  dwilliss
 * Initial revision
 * \endif
 */
#ifndef INC_MI32_MG2X_H
#define INC_MI32_MG2X_H

#ifndef X_NATIVE
#error Cannot use this header unless building for X_NATIVE
#endif

#ifndef INC_MI32_MGD2_H
#include <mi32/mgd2.h>
#endif

#include <X11/Intrinsic.h>	// To define Display and Drawable

namespace MGD {

class CONTEXT_X;

//! An X11-based implementation of MGD::DEVICE 
class DEVICE_X : public DEVICE {

	public:

		//! Construct an X device.
		//! After constructing, you should call Create()
		DEVICE_X();

		// Note: constructor is protected.  Can only construct a derived class

		//! Destructor
		virtual ~DEVICE_X();

		// (Note: copy constructor private and unimplemented.  Can't copy one.)

		//! Takes an XtWindow() of a widget
		ERRVALUE ChangeDrawable (
			Drawable drawable
			);		

		//! Takes a widget and does the same thing as ChangeDrawable using it's
		//! XtWindow.  This however, gets the height and width of the widget
		//! directly from the widget, so it's faster.
		ERRVALUE ChangeWidget (
			Widget widget
			);		

		ERRVALUE Create (
			Display* display,
			Drawable drawable
			);

		Display* GetDisplay () const
			{ return (m_display); }

		Drawable GetDrawable () const
			{ return (m_drawable); }

	protected:

		virtual CAPABILITY v_GetCapabilities () const;

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DEVICE BASECLASS;
		friend class CONTEXT_X;

		//! Copy constructor private and unimplemented
		DEVICE_X (const DEVICE_X&);

		Display *m_display;
		XImage *m_ximage;
		void *m_imagedata;
		Drawable m_drawable;
		UINT8 m_tempbuf[512];	//!< tempory buffer for X protocol what's this for?

		virtual CONTEXT* v_CreateContext ();
		virtual ERRVALUE v_FillSpan (MGD::CONTEXT *gc, INT32 x1, INT32 x2, INT32 y);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

class CONTEXT_X : public CONTEXT {
	public:

		virtual ~CONTEXT_X ();

		GC GetXGC (
			) const {
			return (m_gc);
			}

	protected:

		//! Standard constructor.
		CONTEXT_X (
			DEVICE_X *pDevX
			);

		DEVICE_X *m_DevX;
		
	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef MGD::CONTEXT BASECLASS;

		int m_arcmode;
		GC m_gc;
		Pixmap m_PixmapStipple;		// Pixmap used for stipple

		// Copy constructor private and unimplemented
		CONTEXT_X (const CONTEXT_X&);

		friend class MGD::DEVICE_X;	// Only DEVICE_X::CreateContext() can call our constructor
		
		virtual ERRVALUE v_CopyRect (const MGD::DEVICE *pDevSrc, const LRECT2D& RectSrc, const LPOINT2D& PtTgtIn, MGD::DEVICE_MEM8G *pDevAlpha);
		virtual ERRVALUE v_DrawArc (INT32 xcenter, INT32 ycenter, INT32 xradius, INT32 yradius, double startangle, double sweepangle, double rotangle = 0, MGD::SHAPECLOSE closemode = SHAPECLOSE_None);
		virtual ERRVALUE v_DrawCircle (INT32 xcenter, INT32 ycenter, INT32 radius);
		virtual ERRVALUE v_DrawEllipse (INT32 xcenter, INT32 ycenter, INT32 xradius, INT32 yradius, double rotangle = 0.0);
		virtual ERRVALUE v_DrawIcon (INT32 x, INT32 y, MICONSTRUCT* icon, ICONFLAGS flags);
		virtual ERRVALUE v_DrawLine (INT32 x1, INT32 y1, INT32 x2, INT32 y2);
		virtual ERRVALUE v_DrawLinesLow (const LPOINT2D* points, INT32 numpoints);
		virtual ERRVALUE v_DrawLinesLow (const POLYLINE& line);
		virtual ERRVALUE v_DrawPoint (INT32 x, INT32 y);
		virtual ERRVALUE v_DrawPoints (const LPOINT2D *points, INT32 NumPoints);
		virtual ERRVALUE v_DrawRectangle (INT32 x1, INT32 x2, INT32 y1, INT32 y2);
		virtual ERRVALUE v_DrawSegments (const LSEGMENT2D* segments, INT32 NumSegments);
		virtual ERRVALUE v_FillArc (INT32 xcenter, INT32 ycenter, INT32 xradius, INT32 yradius, double startangle, double sweepangle, double rotangle = 0, MGD::SHAPECLOSE closemode = SHAPECLOSE_Direct);
		virtual ERRVALUE v_FillCircle (INT32 xcenter, INT32 ycenter, INT32 radius);
		virtual ERRVALUE v_FillEllipse (INT32 xcenter, INT32 ycenter, INT32 xradius, INT32 yradius, double rotangle = 0.0);
		virtual ERRVALUE v_FillPolygon (const LPOINT2D* points, INT32 NumPoints, MGD::POLYSHAPE shape = POLYSHAPE_Complex, bool DrawBorder = false);
		virtual ERRVALUE v_FillRectangle (INT32 x1, INT32 x2, INT32 y1, INT32 y2);
		ERRVALUE v_InterfaceTextDrawString (INT32 x, INT32 y, const MIUNICODE* string, TEXTBASEORIGIN origin, bool bVisualOrder, INTERFACETEXTSTYLE style);
		ERRVALUE v_InterfaceTextGetExtents (const MIUNICODE* string, int *pwidth, int *pheight, int *pascent, INTERFACETEXTSTYLE style);
		ERRVALUE v_InterfaceTextGetMetrics (const MIUNICODE* string, TEXTMETRICS& metrics, INTERFACETEXTSTYLE style);
		int v_InterfaceTextGetWidth (const MIUNICODE* string, INTERFACETEXTSTYLE style, bool bInkMetrics);
		virtual ERRVALUE v_PutImageSpanDirect (INT32 x, INT32 y, INT32 len, const void* data, const UINT8* mask = 0);
		virtual ERRVALUE v_PutPixel (INT32 x, INT32 y, MGD::PIXEL pixel);
		virtual void v_UpdateClipNone ();
		virtual void v_UpdateClipRect ();
		virtual void v_UpdateColor ();
		virtual void v_UpdateLineAttributes ();
		virtual void v_UpdateLineWidth ();
		virtual void v_UpdatePixelFunc ();
		virtual void v_UpdatePlaneMask ();
		virtual void v_UpdateStipple ();
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

} // End of namespace MGD


#endif	// INC_MI32_MG2X_H

