/******************************************************************************
 * \file mi32/mg2x32.h
 * \brief An X device with a 32-bit frame buffer.
 *
 * \if NODOC
 * $Log: mg2x32.h_v $
 * Revision 1.11  2012/02/20 18:41:05  mju
 * Derive from mem32rgb.
 *
 * Revision 1.10  2011/10/28 14:47:37  mju
 * Remove obsolete methods and unused shared memory members.
 *
 * Revision 1.9  2005/02/07 18:49:58  mju
 * Add create method to deal with widget, and updateSize.
 *
 * Revision 1.6  2003/06/11 17:12:49  dwilliss
 * Virtual methods all renamed to start with v_ and made private (where
 * possible) or protected.
 *
 * Revision 1.1  2002/02/21 15:46:49  dwilliss
 * Initial revision
 * \endif
 */

#ifndef INC_MI32_MG2X32_H
#define INC_MI32_MG2X32_H

#ifndef INC_MI32_MG2X_H
#include <mi32/mg2x.h>
#endif

#ifndef INC_MI32_MG2MEM32RGB_H
#include <mi32/mg2mem32rgb.h>
#endif

namespace MGD {

//! A hybrid between a DEVICE_X and DEVICE_MEM32RGB.
//!
//! To the caller, this seems to be just like an MGD::DEVICE_MEM32RGB but it
//! also creates an X Pixmap the same size.  After doing some rendering,
//! call UpdatePixmap() to transfer the contents of the memory frame buffer
//! to the pixmap.  If the X device is not 32-bit, it will automatically
//! convert the image to the format necessary.
class DEVICE_X32 : public MGD::DEVICE_MEM32RGB {
	public:

		//! constructor
		DEVICE_X32 ();

		//! Destructor
		virtual ~DEVICE_X32 ();

		//! Create a buffered pixmap for an X Widget.
		//! Size will be determined from widget.
		//! Widget background pixmap will be set to the buffered pixmap.
		//! When widget size changes, you must call UpdateSize() to adjust the
		//! pixmap size.  This is not done automatically to avoid multiple updates
		//! during user sizing operation.
		ERRVALUE Create (
			Widget widget
			);

		//! Create a buffered pixmap for an X display and drawable.
		ERRVALUE Create (
			Display *display,							//!< X display pointer
			Drawable drawable,						//!< X drawable
			INT32 height,								//!< Height in pixels
			INT32 width									//!< Width in pixels
			);

		//! Return the pixmap being used for this device.
		//! Note that the pixmap is owned by the device and will be freed when the device is destroyed.
		Pixmap GetPixmap (
			) const { return (m_pixmap); }

		//! Get the X device associated with this device.
		MGD::DEVICE_X& GetXDevice (
			) { return (m_DevX); }

		//! Resize the pixmap and image buffer.
		//! Current contents are destroyed by this process.  New pixmap
		//! is blank.  Previous pixmap will be destroyed in the process.
		ERRVALUE Resize (
			Drawable drawable,
			INT32 height,
			INT32 width
			) { return (Resize(drawable, height, width, 0)); }

		//! Transfer the contents of the memory frame buffer to the X Pixmap.
		void UpdatePixmap ();

		//! Update buffered pixmap size after widget size changes.
		//! Current pixmap contents will be destroyed and the new contents will
		//! be blank.
		ERRVALUE UpdateSize ();

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT

		// Unimplemented.
		DEVICE_X32 (const DEVICE_X32&);
		DEVICE_X32& operator= (const DEVICE_X32&);

		Display *m_xdisplay;
		Widget m_widget;
		Pixmap m_pixmap;
		DEVICE_X m_DevX;

		static void OnDestroyWidget (Widget, void*, void*);

		void Free ();
		ERRVALUE Resize (Drawable drawable, INT32 height, INT32 width, bool bFromCreate);
				
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

		
}	// End of namespace

#endif	// INC_MI32_MG2X32_H

