/**
 * \file serializeritem.h
 * \brief SERIALIZERITEM definitions
 *
 * \if NODOC
 * $Id: serializeritem.h_v 1.21 2012/11/20 15:21:31 mju Exp $
 *
 * $Log: serializeritem.h_v $
 * Revision 1.21  2012/11/20 15:21:31  mju
 * Add type_DATETIME.
 *
 * Revision 1.20  2012/06/20 21:27:08  mju
 * Adjust dox for type_simple_array to include for struct.
 *
 * Revision 1.19  2012/06/06 14:55:24  mju
 * Remove unused and redundant type_SpecialPtr.
 *
 * Revision 1.18  2012/04/03 16:11:15  mju
 * Add type_rvcobjitemurl.
 *
 * Revision 1.17  2011/09/06 21:33:22  mju
 * Rename macro.
 *
 * Revision 1.16  2010/06/03 14:40:24  mju
 * Add flag_FixedSizeArray.
 *
 * Revision 1.15  2010/02/19 22:18:27  mju
 * Add mergeArrays with offsets.
 *
 * Revision 1.14  2010/02/12 14:35:11  mju
 * Allow type_int32 for type_simple_array.
 *
 * Revision 1.13  2009/03/26 20:15:10  mju
 * Add flag_NoReportMissingFile.
 *
 * Revision 1.12  2008/10/24 17:07:40  mju
 * Update docs as type_int16 supported for simple_array.
 *
 * Revision 1.11  2007/11/16 21:17:03  mju
 * Add type_double_array.
 *
 * Revision 1.10  2007/09/06 21:45:36  mju
 * Make name const.
 *
 * Revision 1.9  2007/09/06 20:57:20  mju
 * Add type_double_range and type_simple_array.
 *
 * Revision 1.8  2007/09/06 17:46:47  mju
 * Remove get/putFilePath methods.
 *
 * Revision 1.7  2007/08/24 21:01:52  mju
 * Remove previous changes, done as new class instead.
 *
 * Revision 1.6  2007/08/24 20:43:44  mju
 * Add merge with simple_array and offset.
 *
 * Revision 1.5  2007/08/24 16:46:28  mju
 * Add rvcobjitem/list types.
 *
 * Revision 1.4  2007/01/25 16:42:44  mju
 * Add type_mistringlist.
 *
 * Revision 1.3  2007/01/08 15:06:47  mju
 * Remove unused types.
 *
 * Revision 1.2  2006/10/31 18:21:35  mju
 * Remove unused type_FileName.
 *
 * Revision 1.1  2006/09/15 15:24:37  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_SERIALIZERITEM_H
#define	INC_MI32_SERIALIZERITEM_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifndef	INC_MI32_SERIALIZERBASE_H
#include <mi32/serializerbase.h>
#endif


//! Serializer Item definition for single member of structure or class.
//!	The meaning of the "extra" field is dependent on the "type" field:
//!
//!		"type"
//!		UINT8_Array						Number of array entries
//!		StringA, StringU				Number of bytes in entry (use membersize in definition)
//!		Object, SubObject				Object type (0 for OTYPE_ALL)
//!		BitUINT32						Bit number to use, 0 = lowest
//!		StructPtr, StructPtrArray	Size of structure to allocate
//!		Enum								Size of member
//!
//! When handling TYPE_Special items, the cbfunc will be called with ACTION_Read or
//! ACTION_Write.  The cbfunc is responsible for actually reading and writing the data
//! using the SERIALIZER class reference provided.
class SERIALIZERITEM {
	public:

		//! Item types.
		enum TYPE {
			TYPE_End =					0,

			// Scalars
			TYPE_UINT8 =				1,
			TYPE_INT8 =					2,
			TYPE_UINT16 =				3,
			TYPE_INT16 =				4,
			TYPE_UINT32 =				5,
			TYPE_INT32 =				6,
			TYPE_BitUINT32 =			7,		//!< Bit value in UINT32, bit specified in "extra"
			TYPE_UInt =					8,		//!< unsigned int
			TYPE_Int =					9,		//!< int
			TYPE_Float =				10,	//!< float
			TYPE_Double =				11,	//!< double
			TYPE_Bool =					12,	//!< bool
			TYPE_UINT8_Array =		13,
			TYPE_Enum =					14,	//!< enum, size specified in "extra"

			// Strings
			TYPE_StringU =				20,	//!< MIUNICODE string ("extra" contains size)
			TYPE_MISTRING =			21,	//!< MISTRING
			TYPE_MISTRING_ML =		22,	//!< Multi-line MISTRING
			TYPE_StringMLU =			23,	//!< Multi-line Unicode string
			TYPE_MISTRINGLIST =		24,	//!< MISTRINGLIST
			TYPE_StringA =				25,	//!< ASCII string ("extra" contains size)
			TYPE_StringMLRaw =		28,	//!< Multi-line raw (char*) string (pointer)

			// Files and objects
			TYPE_FILEPATH =			31,	//!< File path
			TYPE_RVCOBJITEM =			32,	//!< RVC::OBJITEM
			TYPE_RVCOBJITEMLIST =	33,	//!< RVC::OBJITEMLIST
			TYPE_RVCOBJECTNAME =		34,	//!< RVC::OBJECTNAME
			TYPE_RVCOBJITEMFILE =	35,	//!< FilePath portion of RVC::OBJITEM
			TYPE_RVCOBJITEMOBJECT =	36,	//!< Object portion of RVC::OBJITEM, type stored in "extra"
			TYPE_RVCOBJECTFILE =		37,	//!< FilePath portion of RVC::OBJECT
			TYPE_RVCOBJECTOBJECT =	38,	//!< Object portion of RVC::OBJECT, type stored in "extra"
			TYPE_RVCOBJITEMURL =		39,	//!< URL portion of RVC::OBJITEM, type stored in "extra"

			// Generic
			TYPE_Special =				40,	//!< Use cbfunc to do read and write
			TYPE_Struct =				42,	//!< Structure/class
			TYPE_StructPtr =			43,	//!< Structure/class pointer, will be allocated if 0, "extra" contains size
			TYPE_StructPtrArray =	44,	//!< Array of structure pointers, "extra" contains size
			TYPE_SIMPLE_ARRAY =		45,	//!< Simple array of scalars or structs.  If scalar, "extra" contains TYPE of array item, only TYPE_UINT8, TYPE_INT16, TYPE_INT32 supported.  If struct, "extra" contains size and "substruct" points to item definition array.
			TYPE_DOUBLE_ARRAY =		46,	//!< Double-precision array of "double" scalars

			// Built-in structures
			TYPE_COLOR =				50,	//!< COLOR
			TYPE_DRECT2D =				51,	//!< DRECT2D
			TYPE_DPOINT2D =			52,	//!< DPOINT2D
			TYPE_DPOINT3D =			53,	//!< DPOINT3D
			TYPE_POINTSTYLE =			55,	//!< POINTSTYLE
			TYPE_LINESTYLE =			56,	//!< LINESTYLE
			TYPE_POLYSTYLE =			57,	//!< POLYSTYLE
			TYPE_TEXTSTYLE =			58,	//!< TEXTSTYLE
			TYPE_LPOINT2D =			59,	//!< LPOINT2D
			TYPE_LRECT2D =				60,	//!< LRECT2D
			TYPE_OLDPROJPARMTOCRS =	61,	//!< Serialized as OLDPROJPARM, in memory as SPATREF::COORDREFSYS
			TYPE_COORDREFSYS =		62,	//!< SPATREF::COORDREFSYS
			TYPE_REGION2D =			63,	//!< REGION2D
			TYPE_DRECT3D =				64,	//!< DRECT3D
			TYPE_DOUBLE_RANGE =		65,	//!< DOUBLE_RANGE
			TYPE_DATETIME =			66,	//!< DATETIME

			TYPE_SERIALIZABLE =		90,	//!< SERIALIZABLE object
			TYPE_SERIALIZABLEPTR =	91,	//!< Pointer to SERIALIZABLE object

			TYPE_InternalSer =		998,
			TYPE_Start =				999	//!< Used internally to start new structure
			};

		enum FLAGS {
			FLAG_None =						0x0000,
			FLAG_NoWrite0 =				0x0001,	//!< Don't write value if not set
			FLAG_NoWrite =					0x0002,	//!< Don't write this item
			FLAG_NoRead =					0x0004,	//!< Don't read this item
			FLAG_FileNoExist0 =			0x0008,	//!< When reading PARMITEM_FileName zero string if file doesn't exist
			FLAG_NoReportMissingFile =	0x0010,	//!< Don't report missing file and leave path as read
			FLAG_Skippable =				0x0020,	//!< Item will be ignored if "IgnoreSkippable" set
			FLAG_AddOffsetCB =			0x0040,	//!< Add offset to pointer when passing to callback function
			FLAG_FindFileImmed =			0x0080,	//!< Search for file immediately instead of waiting until looking for object
			FLAG_DontAskLinkFile =		0x0100,	//!< Don't prompt user if can't find linked file
			FLAG_ArrayNumItems =			0x0200,	//!< This is the "NumItems" for an array that follows
			FLAG_FixedSizeArray =		0x0400,	//!< Fixed-size array, "extra" contains number of elements (for simple and some built-in types only, typically with < 256 elements)
			};

		enum ACTION {
			ACTION_None =				0x0000,
			ACTION_ReadBegin =		0x0001,
			ACTION_ReadEnd =			0x0002,
			ACTION_WriteBegin =		0x0004,
			ACTION_WriteEnd =			0x0008,
			ACTION_Read =				0x0010,
			ACTION_Write =				0x0020
			};

		typedef ERRVALUE (*CBFUNC)(SERIALIZER&, const SERIALIZERITEM*, void *data, ACTION action);

		const char *name;						//!< Name of the item, 0 to terminate array, not case-sensitive
		TYPE type;								//!< Item type
		int offset;								//!< Offset into class/struct
		int extra;								//!< Extra type-specific data, see TYPE descriptions for usage
		const SERIALIZERITEM *substruct;	//!< Sub-structure definition for TYPE_Struct/StructPtr
		FLAGS flags;							//!< Flags
		CBFUNC cbfunc;							//!< Callback function for this item

		//! Merge two item definition arrays.
		//! This is useful when inheriting from a class which already uses an item definition array.
		//! @return New allocated definition array
		static SERIALIZERITEM* MergeArrays (
			const SERIALIZERITEM *def1,		//!< First definition array
			const SERIALIZERITEM *def2			//!< Second definition array
			);

		//! Merge two item definition arrays with offsets,
		//! This is useful when inheriting from a class which already uses an item definition array.
		//! @return New allocated definition array
		static SERIALIZERITEM* MergeArrays (
			const SERIALIZERITEM *def1,		//!< First definition array
			int offset1,							//!< Offset for first definition array
			const SERIALIZERITEM *def2,		//!< Second definition array
			int offset2								//!< Offset for second definition array
			);

	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(SERIALIZERITEM::FLAGS)
DEFINE_ENUM_OP_BITWISE(SERIALIZERITEM::ACTION)
#endif

#endif	// INC_MI32_SERIALIZERITEM_H
