/******************************************************************************
 *
 * \file mi32/soap.h
 * \brief SOAP - Simple Object Access Protocol -- Client-Side
 *
 * \if NODOC
 * $Log: soap.h_v $
 * Revision 1.2  2004/01/20 15:02:58  dwilliss
 * Added some methods
 *
 * Revision 1.1  2004/01/08 16:39:49  dwilliss
 * Initial revision
 *
 * \endif
 *
 *****************************************************************************/

#ifndef INC_MI32_SOAP_H
#define INC_MI32_SOAP_H


#ifndef INC_MI32_XML_H
#include <mi32/xml.h>
#endif

#ifndef INC_MI32_HTTPCLIENT_H
#include <mi32/httpclient.h>
#endif


namespace SOAP {

//! A class to hold the actual parameters and return values of a SOAP
//! message.  This class is used on by both the caller and the callee.
class VALUES {
	public:
		enum TYPE {
			TYPE_String,
			TYPE_Double,
			TYPE_Integer,
			TYPE_Bool,
			TYPE_XML
			};

		VALUES();

		// Parse the values out of the body of an HTTP message
		// Content-Type of the message should be either text/xml or
		// application/x-www-form-urlencoded (which is how HTML form data
		// is transmitted via POST or GET)
		VALUES(const HTTPMESSAGE& message);

		~VALUES();

		void Clear();

		//! Get a parameter or return value (MISTRING)
		//! \return true if the parameter was passed, false if not
		bool GetValue(const char* name, MISTRING& value) const;

		//! Get a parameter or return value (STRUTF8)
		//! \return true if the parameter was passed, false if not
		bool GetValue(const char* name, STRUTF8& value) const;

		//! Get a parameter or return value (double)
		//! \return true if the parameter was passed, false if not
		bool GetValue(const char* name, double& value) const;

		//! Get a parameter or return value (INT32)
		//! \return true if the parameter was passed, false if not
		bool GetValue(const char* name, INT32& value) const;

		//! Get a parameter or return value (UINT32)
		//! \return true if the parameter was passed, false if not
		bool GetValue(const char* name, UINT32& value) const;

		//! Get a parameter or return value (bool)
		//! \return true if the parameter was passed, false if not
		bool GetValue(const char* name, bool& value) const;

		//! Get a parameter or return value's XMLNODE
		//! This is useful mainly for items which are complex structures
		//! not directly supported by this class
		//! \return An XMLNODE* or NULL if not found.  
		XMLNODE* GetValue (const char* name) const;

		//! Determine if a value exists
		bool HasValue (const char* name) const;

		//! Set a parameter or return value (MISTRING)
		void SetValue(const char* name, const MISTRING& value);

		//! Set a parameter or return value (STRUTF8)
		void SetValue(const char* name, const STRUTF8& value);

		//! Set a parameter or return value (double)
		void SetValue(const char* name, double value);

		//! Set a parameter or return value (INT32)
		void SetValue(const char* name, INT32 value);

		//! Set a parameter or return value (UINT32)
		void SetValue(const char* name, UINT32 value);

		//! Set a parameter or return value (bool)
		void SetValue(const char* name, bool value);

		//! Set a parameter or return value (XMLNODE)
		//! This is mainly for cases where a parameter or value is a complicated
		//! structure not directly supported by this class.  
		//! \param name The name of the value to set
		//! \param node The XML node to set it from
		void SetValue (
			const char* name, 
			XMLNODE* node
			);

		//! Return the XMLNODE containing all the values
		XMLNODE* GetXMLNode (
			) const;

		//! Sets the root node.  Makes a copy of the node pased in.
		void SetXMLNode (
			XMLNODE* node
			);
	
	private:
		// Construct given an XML node
		VALUES (
			XMLNODE* node
			);

		XMLNODE *m_node;
		XMLDOC *m_doc;

		friend class METHOD;	// Server side method defn can construct one
		friend class HANDLER;	// The server-side handler can construct one
	};


//! Client-Side SOAP Object container
class OBJECT {
	public:
		//! Construct an object but don't bind it to anything
		OBJECT();

		//! Construct an object and bind it to a client.
		//! Note: this constructor may become protected.
		OBJECT (
			HTTPCLIENT* client,
			const char* ObjectClass
			);

		virtual ~OBJECT();

		//! Allocate a server-side instance of the object.
		//! If the object is conceptually an interface-only object and has all
		//!	static method (in the C++ sense of the word), then you do not need
		//! 	to allocate an instance.  In fact, trying to do so will probably
		//!   just give you an error.
		//! \return 0 if attach successful, error code < 0 if not.
		ERRVALUE Allocate (
			);
		
		//! Attach this OBJECT to a client and object class
		void Attach (
			HTTPCLIENT* client,	// XXX May want to define a SOAPCLIENT
			const char* ObjectClass
			);
		
		//! Call a method on the object.
		ERRVALUE CallMethod (
			const char* name,
			const VALUES* parms = 0,
			VALUES* ret = 0
			);
	
		void Free (
			);

		// Get the object's ID. 
		// An object's ID is like a pointer, but identifies where the object
		// in in the server.
		const char* GetObjectID (
			) const;

	protected:

		HTTPCLIENT* GetClient(
			) const;

	private:
		STRUTF8 m_id;
		STRUTF8 m_ObjClass;
		HTTPCLIENT* m_client;

	};

}		// End of SOAP namespace

#endif // INC_MI32_SOAP_H

