/**
 * \file <mi32/stereodevice.h>
 * \brief STEREODEVICE definitions
 *
 * \if NODOC
 * $Id: stereodevice.h_v 1.10 2012/03/23 20:14:13 mju Exp $
 *
 * $Log: stereodevice.h_v $
 * Revision 1.10  2012/03/23 20:14:13  mju
 * Add isValid.
 *
 * Revision 1.9  2012/03/23 17:52:51  mju
 * Add method_Unspecified and default for parm in dlgEditValues.
 *
 * Revision 1.8  2012/03/01 15:55:22  mju
 * Add method_OpenGL.
 *
 * Revision 1.7  2011/12/19 16:46:55  mju
 * Add getInterlaceStartRight.
 *
 * Revision 1.6  2011/12/19 15:48:45  mju
 * Add getStippleMasks.
 *
 * Revision 1.5  2011/09/28 21:26:56  mju
 * Add interlace / split modes and split method.
 * Rename as stereodevice from stereoparm.
 *
 * Revision 1.4  2007/05/07 20:21:56  mju
 * Check use_mgui for dialog.
 *
 * Revision 1.3  2007/05/07 17:55:08  mju
 * Add allowSeparate parm to dlg.
 *
 * Revision 1.2  2007/05/04 21:40:41  mju
 * Add inigroup parm.
 *
 * Revision 1.1  2007/04/18 17:24:02  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_STEREODEVICE_H
#define	INC_MI32_STEREODEVICE_H

#ifndef INC_MI32_STDDEFNS_H
	#include <mi32/stddefns.h>
#endif


#ifndef	GENERATING_DOXYGEN_OUTPUT
class STIPPLEMASK;
#endif


//! Stereo device parameters.
class STEREODEVICE {
	public:

		//! Basic display method.
		enum METHOD {
			METHOD_Unspecified =			0,
			METHOD_SeparateFrames =		100,
			METHOD_InterlaceLines =  	101,
			METHOD_InterlaceColumns =	102,
			METHOD_Anaglyph =				103,
			METHOD_Sharp =					104,
			METHOD_Split =					105,	//!< Split window, full frame reduced to fit half window, and subsequently stretched by displayed hardware
			METHOD_OpenGL =				106,	//!< Use OpenGL if available, first rendering to separate frames
			};

		//! Mode for anaglyph method.
		enum ANAGLYPHMODE {
			ANAGLYPHMODE_RedGreen =			0,
			ANAGLYPHMODE_RedBlue =			1,
			ANAGLYPHMODE_RedCyan =			2,
			ANAGLYPHMODE_GreenRed =			3,
			ANAGLYPHMODE_GreenBlue =		4,
			ANAGLYPHMODE_GreenMagenta =	5,
			ANAGLYPHMODE_BlueRed =			6,
			ANAGLYPHMODE_BlueGreen =		7,
			ANAGLYPHMODE_BlueYellow =		8,
			ANAGLYPHMODE_CyanRed =			9,
			ANAGLYPHMODE_MagentaGreen =	10,
			ANAGLYPHMODE_YellowBlue =		11,
			ANAGLYPHMODE_RedBlueMI =		12		//!< MicroImages red-blue glasses 
			};

		//! Modes for interlace methods.
		enum INTERLACEMODE {
			INTERLACEMODE_LeftEven =		0,		//!< Left view is on even column/row (right on odd)
			INTERLACEMODE_RightEven =		1		//!< Right view is on even column/row (left on odd)
			};

		//! Modes for "split" method.
		enum SPLITMODE {
			SPLITMODE_LeftRight =			0,
			SPLITMODE_RightLeft =			1,
			SPLITMODE_LeftAboveRight =		2,
			SPLITMODE_RightAboveLeft =		3
			};

		//! Constructor.
		STEREODEVICE ();

	#ifdef USE_MGUI
		//! Dialog for editing values.
		//! @return 0 if OK or EUserCancel
		int DlgEditValues (
			MDLGPARENT dlgparent,
			const char *inigroup,
			bool AllowSeparateFrames = false
			);
	#endif

		ANAGLYPHMODE GetAnaglyphMode () const
			{ return (m_AnaglyphMode); }

		INTERLACEMODE GetInterlaceMode () const
			{ return (m_InterlaceMode); }

		bool GetInterlaceStartRight (
			INT32 OriginX = 0,						//!< Canvas origin X on screen
			INT32 OriginY = 0							//!< Canvas origin Y on screen
			) const;

		void GetMaskLeft (
			UINT8& MaskRed,
			UINT8& MaskGreen,
			UINT8& MaskBlue
			) const;

		void GetMaskRight (
			UINT8& MaskRed,
			UINT8& MaskGreen,
			UINT8& MaskBlue
			) const;

		METHOD GetMethod () const
			{ return (m_Method); }

		double GetMonitorSeparation () const
			{ return (m_MonitorSeparation); }

		double GetOpticalSeparation () const
			{ return (m_OpticalSeparation); }

		SPLITMODE GetSplitMode () const
			{ return (m_SplitMode); }

		//! Get stipple masks for interlaced modes.
		void GetStippleMasks (
			STIPPLEMASK& MaskLeft,					//!< Left mask returned
			STIPPLEMASK& MaskRight,					//!< Right mask returned
			INT32 OriginX = 0,						//!< Canvas origin X on screen
			INT32 OriginY = 0							//!< Canvas origin Y on screen
			) const;

		bool IniRead (
			const char *inigroup,					//!< Ini group
			INIHANDLE inihnd = 0						//!< Ini Handle created by IniOpen(), default if 0
			);

		void IniWrite (
			const char *inigroup,					//!< Ini group
			INIHANDLE inihnd = 0						//!< Ini Handle created by IniOpen(), default if 0
			) const;

		bool IsEqual (
			const STEREODEVICE rhs
			) const;

		//! Determine if current settings are valid.
		bool IsValid () const;

		void SetAnaglyphMode (
			ANAGLYPHMODE AnaglyphMode
			) { m_AnaglyphMode = AnaglyphMode; }

		void SetInterlaceMode (
			INTERLACEMODE InterlaceMode
			) { m_InterlaceMode = InterlaceMode; }

		void SetMethod (
			METHOD Method
			) { m_Method = Method; }

		void SetMonitorSeparation (
			double MonitorSeparation				//!< Monitor separation in millimeters
			) { m_MonitorSeparation = MonitorSeparation; }

		void SetOpticalSeparation (
			double OpticalSeparation				//!< Optical separation in millimeters
			) { m_OpticalSeparation = OpticalSeparation; }

		void SetSplitMode (
			SPLITMODE SplitMode
			) { m_SplitMode = SplitMode; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		double m_OpticalSeparation;
		double m_MonitorSeparation;
		METHOD m_Method;
		ANAGLYPHMODE m_AnaglyphMode;
		INTERLACEMODE m_InterlaceMode;
		SPLITMODE m_SplitMode;

		#endif
	};

inline bool operator== (const STEREODEVICE& lhs, const STEREODEVICE& rhs)
	{ return (lhs.IsEqual(rhs)); }

inline bool operator!= (const STEREODEVICE& lhs, const STEREODEVICE& rhs)
	{ return (!lhs.IsEqual(rhs)); }


#endif	// INC_MI32_STEREODEVICE_H
