/**
 * \file <mi32/tinrc16.h>
 * \brief TIN_RC16 definitions
 *
 * \if NODOC
 * $Id: tinrc16.h_v 1.2 2010/09/10 15:58:03 mju Exp $
 *
 * $Log: tinrc16.h_v $
 * Revision 1.2  2010/09/10 15:58:03  mju
 * Add doRowNodes.
 *
 * Revision 1.1  2010/09/09 21:21:47  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_TINRC16_H
#define	INC_MI32_TINRC16_H

#ifndef	INC_MI32_DELEGATE_H
#include <mi32/delegate.h>
#endif

#ifndef	INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif

#ifndef	INC_MI32_DOUBLEAR_H
#include <mi32/doublear.h>
#endif

#ifndef	INC_MI32_POINT_H
#include <mi32/point.h>
#endif

//=====================================================================================================================

//! Triangulated Irregular Network using 16-bit row/col coordinates.
class TIN_RC16 {
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		
		struct EDGE {
			INT32 m_NodeStart;
			INT32 m_NodeEnd;
			INT32 m_TriangleLeft;
			INT32 m_TriangleRight;
			UINT32 m_Length2;
			};

		struct TRIANGLE {
			INT32 m_Node[3];
			INT32 m_Edge[3];

			int GetNodePosOpposite (INT32 EdgeIdx) const
				{ return ((m_Edge[1] == EdgeIdx) ? 0 : (m_Edge[2] == EdgeIdx) ? 1 : 2); }
			};

		#endif

	public:

		//! Node with 16-bit row/col coordites.
		struct NODE {
			INT16 m_row;
			INT16 m_col;
			};

		struct PLANE {
			double m_Zrow;
			double m_Zcol;
			double m_Z0;
			};

		//! Container for 3 nodes defining triangle face.
		struct TRINODE {
			NODE m_Node[3];

			//! Calculate unit normal vector.
			void CalcNormal (
				const double *values,	//!< Array of 3 values corresponding to 3 node positions
				DPOINT3D& normal			//!< Normal vector returned, x=row, y=col, z=value
				) const;

			//! Calculate plane coefficients.
			void CalcPlane (
				const double *values,	//!< Array of 3 values corresponding to 3 node positions
				PLANE& plane				//!< Plane definition returned
				) const;
			};

		//! Base class for filling all faces in TIN.
		class FILLER {
			public:
				//! Constructor.
				FILLER (const TIN_RC16& tin);

				//! Destructor.
				virtual ~FILLER ();

				//! Process all triangles.
				//! Assumes that nodes are already filled and thus will not be filled again,
				//! which will result in half-cell triangles being skipped.
				void Process ();

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				const TIN_RC16& m_tin;

				void FillTriangle (const TRIANGLE& triangle);
				#endif

				// Overridables.

				virtual void v_FillBegin (
					const TRINODE& trinode
					);

				virtual void v_FillSpan (
					INT16 row,
					INT16 col, 
					UINT16 num
					) = 0;
			};

		//! Filler for double-precision values in memory array.
		class FILLER_MEM_DOUBLE : public FILLER {
			public:

				//! Constructor.
				FILLER_MEM_DOUBLE (
					const TIN_RC16& tin,
					double *data,
					UINT16 NumRows,
					UINT16 NumCols
					);

				//! Destructor.
				~FILLER_MEM_DOUBLE ();

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				double *m_data;
				INT32 m_NumRows;
				INT32 m_NumCols;
				PLANE m_Plane;

				INT32 CalcIdx (INT16 row, INT16 col) const
					{ return (row * m_NumCols + col); }

				virtual void v_FillBegin (const TRINODE& trinode);
				virtual void v_FillSpan (INT16 row, INT16 col, UINT16 num);
				#endif
			};

		//!< Delegate to get value for specified cell (row, column).
		typedef fastdelegate::FastDelegate<double(INT16,INT16)> DELEGATE_GETVALUE;

		//!< Constructor.
		TIN_RC16 ();

		//!< Destructor.
		~TIN_RC16 ();

		//! Add all cells for specific row.
		ERRVALUE AddRowCells (
			INT16 row,										//!< Raster row, rows must be added in increasing order.
			const INT16 *columns,						//!< All columns for row, must be in increasing order
			UINT32 NumColumns								//!< Number of columns in row
			);

		//! Calculate normal vectors at nodes.
		ERRVALUE CalcNodeNormals (
			DELEGATE_GETVALUE DelegateGetValue,		//!< Delegate to get Z value for 
			double rowscale,
			double colscale,
			DOUBLE_ARRAY<DPOINT3D>& NodeNormals		//!< Normals corresponding to nodes returned
			) const;

		//! Finish filling remaining cells.
		//! Must be called after all row cells are added.
		ERRVALUE Finalize ();

		//! Get array of nodes.
		const SIMPLE_ARRAY<NODE>& GetNodes () const
			{ return (m_Nodes); }

		//! Initialize.
		//! Must be called before AddRowCells.
		ERRVALUE Initialize ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		INT32 m_RowPrev;
		SIMPLE_ARRAY<NODE> m_Nodes;
		SIMPLE_ARRAY<EDGE> m_Edges;
		SIMPLE_ARRAY<INT32> m_EdgeIdxTop;
		SIMPLE_ARRAY<INT32> m_EdgeIdxBottom;
		SIMPLE_ARRAY<INT32> m_EdgeIdxWork;
		SIMPLE_ARRAY<INT32> m_EdgeIdxTest;
		SIMPLE_ARRAY<INT32> m_TriangleIdxTest;
		SIMPLE_ARRAY<TRIANGLE> m_Triangles;

		ERRVALUE DoRowNodes (INT16 row, INT32& NodeIdx);

		INT32 EdgeAdd (INT32 NodeIdx1, INT32 NodeIdx2);

		void EdgeTestAdd (INT32 EdgeIdx, INT32 TriangleIdx);

		int EdgeTestCell (const EDGE& Edge, INT16 row, INT16 col) const
			{ return (NodesTestCell(Edge.m_NodeStart,Edge.m_NodeEnd,row,col)); }

		int EdgeTestCell (INT32 EdgeIdx, INT16 row, INT16 col) const
			{ return (EdgeTestCell(m_Edges[EdgeIdx],row,col)); }

		void EdgeTestResplit (INT32 EdgeIdx, INT32 TriangleIdx);

		void EdgeTestSliver (INT32 EdgeIdx, SIMPLE_ARRAY<INT32>& EdgeIdxTest);

		INT32 NodeAdd (INT16 row, INT16 col);

		UINT32 NodeCalcDistance2 (INT32 NodeIdx1, INT32 NodeIdx2) const
			{
			INT32 drow = m_Nodes[NodeIdx2].m_row - m_Nodes[NodeIdx1].m_row;
			INT32 dcol = m_Nodes[NodeIdx2].m_col - m_Nodes[NodeIdx1].m_col;
			return (drow*drow + dcol*dcol);
			}

		int NodesTestCell (INT32 NodeIdxStart, INT32 NodeIdxEnd, INT16 row, INT16 col) const
			{ return (NodesTestCell(m_Nodes[NodeIdxStart],m_Nodes[NodeIdxEnd],row,col)); }

		int NodesTestCell (const NODE& NodeStart, const NODE& NodeEnd, INT16 row, INT16 col) const
			{
			INT32 drows = NodeEnd.m_row - NodeStart.m_row;
			INT32 dcols = NodeEnd.m_col - NodeStart.m_col;
			INT32 trows = row - NodeStart.m_row;
			INT32 tcols = col - NodeStart.m_col;
			return (dcols * trows - drows * tcols);	// < 0 if left, > 0 if right
			}

		ERRVALUE TriangleAdd (INT32 NodeIdx0, INT32 NodeIdx1, INT32 NodeIdx2, INT32 EdgeIdx01, INT32 EdgeIdx12, INT32 EdgeIdx20);
		bool TriangleTestNodeInCircle (const TRIANGLE& Triangle, const NODE& Node) const;

	#if defined(WIN32) && defined(DEBUG)
		void TriangleAssertValid (INT32 TriangleIdx) const;
	#endif

		TIN_RC16 (const TIN_RC16&);
		TIN_RC16& operator= (const TIN_RC16&);

		#endif
	};

//=====================================================================================================================

#endif	// INC_MI32_TINRC16_H
