/**
 * \file rvc/array.h 
 * \brief interface for the RVCARRAY class.
 *
 * \if NODOC
 * $Id: array.h_v 1.21 2007/03/30 22:51:12 scowan Exp $
 *
 * $Log: array.h_v $
 * Revision 1.21  2007/03/30 22:51:12  scowan
 * Put array info in rvc namespace.
 *
 * Revision 1.20  2006/12/05 15:10:49  scowan
 * Fixed manditory credential error.
 *
 * Revision 1.19  2006/06/28 15:55:56  scowan
 * Changed make to take a non-const objitem.
 *
 * Revision 1.18  2006/01/10 15:32:13  scowan
 * Added inline template method.
 *
 * Revision 1.17  2005/12/19 17:02:23  linux32build!dwilliss
 * Credentials can now be made mandatory by setting a #define
 *
 * Revision 1.16  2005/12/09 17:29:08  scowan
 * Updated to use credentials.
 *
 * Revision 1.15  2005/07/08 21:13:01  scowan
 * Support use of objMakeParms.
 *
 * Revision 1.14  2005/06/17 17:46:45  scowan
 * Removed deprecated make method.
 *
 * Revision 1.13  2005/02/25 15:18:45  scowan
 * Forgot to check in this include file.
 *
 * Revision 1.12  2005/01/19 22:32:23  scowan
 * Added support for descriptor.
 *
 * Revision 1.11  2004/04/12 19:43:39  scowan
 * Moved open object to protected status.
 *
 * Revision 1.10  2004/04/12 16:41:15  scowan
 * Added is inverted method.
 *
 * Revision 1.9  2003/10/07 17:35:42  scowan
 * Object virtual method changes.
 *
 * Revision 1.8  2003/09/30 16:52:06  dwilliss
 * Doxygen
 *
 * Revision 1.7  2003/09/25 21:20:04  dwilliss
 * Fixed doxygen comment
 *
 * Revision 1.6  2003/09/15 13:49:45  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.5  2002/12/06 15:40:59  scowan
 * Fixed for genator docs.
 *
 * Revision 1.4  2001/06/14 16:03:44  scowan
 * Changed source type.
 *
 * Revision 1.3  2001/06/13 21:25:50  scowan
 * Added rvc to method parameters.
 *
 * Revision 1.2  2001/06/13 19:43:43  scowan
 * Placed into RVC namespace.
 * Added make method.
 *
 * Revision 1.1  2001/06/07 15:39:41  scowan
 * Initial revision
 *
 * \endif
**/

//! \class RVC::ARRAY rvc/array.h
//!	\brief  Manages OBJTYPE_GenericRef RVC objects.  
//!
//!	The interface currently wraps the MfArray functions.
//!	
//!	RVC Object Types that this class supports: OBJTYPE_GenericRef.
//!
//!	Default RVC Object Type for this class: OBJTYPE_GenericRef.
//!
//!	Valid parents for this class:
//!		- OBJTYPE_File
//!		- OBJTYPE_Folder


#ifndef INC_RVC_ARRAY_H
#define INC_RVC_ARRAY_H

#ifndef INC_RVC_OBJECT_H
	#include <rvc/object.h>
#endif

namespace RVC {

#ifndef GENERATING_DOXYGEN_OUTPUT
class ARRAYHANDLE;		// Forward declarations
struct ARRAYINFO;
class OBJMAKEPARMS;
#endif // GENERATING_DOXYGEN_OUTPUT

class ARRAY : public RVC::OBJECT {
	public:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class MAKEPARMS;
		#endif // GENERATING_DOXYGEN_OUTPUT
		
		//! Default constructor
		ARRAY (
			);

		//! Copy constructor
		ARRAY (
			const RVC::ARRAY& obj
			);

		//! Destructor
		virtual ~ARRAY (
			);

		//! Assignment.  Will close if necessary and reopen if 'rhs' is open
		RVC::ARRAY& operator= (
			const RVC::ARRAY& rhs
			);

		//! Append an item to the ARRAY
		//! @return Index to the appended item in the ARRAY
		template <class _CT>
		INT32 Append (
			const _CT& item
			) {
			if (sizeof(_CT) != m_ItemSize) return (EBadFuncParm);
			INT32 retval = GetNumItems();
			ERRVALUE err;
			if ((err = Resize(retval + 1)) < 0) return (err);
			if ((err = Write(retval, &item)) < 0) return (err);
			return (retval);
			}

		//! Obtain the size of a array entry
		//! @return List entry size	
		INT32 GetItemSize (
			) const { return (m_ItemSize); }
			
		//! Obtain the number of array entries
		//! @return Number of array entries
		INT32 GetNumItems (
			) const;

		//! Determine whether or not object is byte order inverted
		//! @return "True' if the object need to be inverted (SwapBytes())			
		bool IsInverted (
			) const;
		
		//! Create a new rvc array object under a specified parent rvc object with name, description, and source
		ERRVALUE Make (
			const RVC::OBJECT& parent,			//!< Parent object to create array object under
			const RVC::DESCRIPTOR& Descriptor,	//!< Name of new array object
			const MAKEPARMS& MakeParms,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Create a new rvc array object under a specified parent rvc object RVC::OBJITEM
		ERRVALUE Make (
			RVC::OBJITEM& ObjItem,						//!< Parent object and DESCRIPTOR of object to create, new location RETURNED
			const MAKEPARMS& MakeParms,
#ifdef RVC_CREDENTIALS_MANDATORY
			const char* Source,							//!< Source of new array object, default is application name
			const CREDENTIALS* Credentials
#else
			const char* Source = 0,						//!< Source of new array object, default is application name
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Read a array entry
		//! @return Number of bytes of the entry		
		INT32 Read (
			INT32 ItemNum,
			void *ptr
			) const;

		//! Resize the array to a new number of items
		ERRVALUE Resize (
			INT32 NewNumItems
			);

		//! Write data to a array item
		//! @return Number of bytes written
		INT32 Write (
			INT32 ItemNum,
			const void* ptr
			);

	protected:		
		
		//! Used for objects that inherit from this object
		ERRVALUE Make (
			const RVC::OBJECT& parent,
			OBJMAKEPARMS& ObjMakeParms
			);

		//! Used for objects that inherit from this object
		ERRVALUE OpenObject (					//! Open the raster object
			OPENMODE OpenFlags,
			ARRAYINFO& newinfo,
			int HeaderSize,
			int HeaderType,
			MDLGPARENT parent,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);
			
	protected:
	
		virtual ERRVALUE v_OpenObject (OPENMODE OpenFlags, MDLGPARENT parent, const CREDENTIALS* Credentials);				//! Open the object
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		ARRAYHANDLE* m_ArrayHandle;
		INT32 m_ItemSize;
		
		//! Implementation of RVC::OBJECT virtual methods
		virtual ERRVALUE v_CloseObject ();				//! Close the open object
		virtual RVC::OBJTYPE v_GetDftObjectType () const;
		virtual bool v_IsObjectOpen () const;			//!< Is this object opened?
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//! Base class for ARRAY MAKEPARMS
class ARRAY::MAKEPARMS {
	public:
		explicit MAKEPARMS (
			INT32 ItemSize
			) : m_ItemSize(ItemSize) {}
				
		INT32 GetItemSize (
			) const { return (m_ItemSize); }
				
		void SetItemSize (
			INT32 ItemSize
			) { m_ItemSize = ItemSize; }
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		INT32 m_ItemSize;	
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
		
}		//! End of RVC namespace

#endif 	//!< INC_RVC_ARRAY_H
