/**
 * \file rvc/getobjfilter.h 
 * \brief RVC::GETOBJFILTER class definitions
 *
 * \if NODOC
 *
 * $Id: getobjfilter.h_v 1.2 2006/10/13 19:26:54 mju Exp $
 *
 * $Log: getobjfilter.h_v $
 * Revision 1.2  2006/10/13 19:26:54  mju
 * Use mistring hdr.
 *
 * Revision 1.1  2006/10/13 17:50:57  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_GETOBJFILTER_H
#define	INC_RVC_GETOBJFILTER_H

#ifndef	INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif

namespace RVC {

class OBJITEM;
class OBJITEMLIST;

//===================================================================================================================
//! \brief Base class for object selection filter.
//! If a filter is provided to the DlgGetObject... function the IsSelectable() method
//! will be called when the user attempts to select an object.  In addition, for the
//! DlgGetObjects() and DlgGetObjectSet() functions, the IsValidList() method will be
//! called when the user presses the 'OK' button.  This allows the list to be checked
//! for self-consistency (for example, to make sure all rasters have equal dimensions).
class GETOBJFILTER {
	public:

		//! Default constructor.
		GETOBJFILTER (
			);

		//! Clear current TEXTID and error value settings.
		void Clear (
			) { m_TextID = TEXTID__None; m_ErrValue = 0; }

		//! Get current error value.
		ERRVALUE GetErrValue (
			) const { return (m_ErrValue); }
			
		//! Get message based on the stored TEXTID and error value.
		//! If both and error value and a TEXTID are set the error value will take precedence.
		MISTRING GetMessage (
			TEXTID TextID = TEXTID__None		//!< Text ID to use for message if none previously set
			) const;
			
		//! Get current TEXTID setting.
		TEXTID GetTextID (
			) const { return (m_TextID); }
			
		//! Determine if has selectable subobjects.
		//! Calls v_HasSelectableSubObjects which can be overridden.
		//! If pFilterProxyFor is not NULL and the virtual method returns false the TextID
		//! and ErrValue will be subsequently set in the specified filter.
		bool HasSelectableSubObjects (
			const RVC::OBJITEM& objitem,			//!< Object to check
			GETOBJFILTER *pFilterProxyFor = 0	//!< Filter this is a proxy for
			);

		//! Determine if object is selectable.
		//! @return 'true' if selectable, 'false' if not.
		//! Calls v_IsSelectable() which can be overridden.
		//! If pFilterProxyFor is not NULL and the virtual method returns false the TextID
		//! and ErrValue will be subsequently set in the specified filter.
		bool IsSelectable (
			const RVC::OBJITEM& objitem,			//!< Object to check
			int position = -1,						//!< Position of the object in the set for DlgGetObjectSet() functions.
			GETOBJFILTER *pFilterProxyFor = 0	//!< Filter this is a proxy for
			);

		//! Determine if list of objects is valid and usable together.
		//! @return 'true' if list is valid, 'false' if not.
		//! Calls v_IsValidList which can be overridden.
		//! If pFilterProxyFor is not NULL and the virtual method returns false the TextID
		//! and ErrValue will be subsequently set in the specified filter.
		bool IsValidList (
			const RVC::OBJITEMLIST& objlist,		//!< Object list to check
			GETOBJFILTER *pFilterProxyFor = 0	//!< Filter this is a proxy for
			);

		//! Determine if the "New Object" button should be enabled.
		//! @return 'true' if parent is valid parent for creating object under.
		//! Calls v_IsValidPArent which can be overridden.
		//! If pFilterProxyFor is not NULL and the virtual method returns false the TextID
		//! and ErrValue will be subsequently set in the specified filter.
		bool IsValidParent (
			const RVC::OBJITEM& parent,			//!< Parent object item
			GETOBJFILTER *pFilterProxyFor = 0	//!< Filter this is a proxy for
			);

		//! Set error value.
		void SetErrValue (
			ERRVALUE err
			) { m_ErrValue = err; }
			
		//! Set the current TEXTID for this filter
		void SetTextID (
			TEXTID TextID
			) { m_TextID = TextID; }
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		TEXTID m_TextID;
		ERRVALUE m_ErrValue;
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Called by HasSelectableSubObjects() to determine if has selectable subobjects.
		//! Default implementation always returns 'false'.
		virtual bool v_HasSelectableSubObjects (
			const RVC::OBJITEM& objitem		//!< Object to check
			);

		//! Called by IsSelectable() to determine if object is selectable.
		//! @return 'true' if selectable, 'false' if not.
		//! Default implementation always returns 'true'.
		virtual bool v_IsSelectable (
			const RVC::OBJITEM& objitem,		//!< Object to check
			int position							//!< Position of the object in the set for DlgGetObjectSet() functions.
			);

		//! Called by IsValidList() to determine if list of objects is valid and usable together.
		//! @return 'true' if list is valid, 'false' if not.
		//! Default implementation always returns 'true'.
		virtual bool v_IsValidList (
			const RVC::OBJITEMLIST& objlist
			);

		//! Called by IsValidParent() to determine if the "New Object" button should be enabled.
		//! @return 'true' if parent is valid parent for creating object under.
		//! Default implementation always returns 'true'.
		virtual bool v_IsValidParent (
			const RVC::OBJITEM& parent
			);

	};


//===================================================================================================================

}	// End namespace RVC

#endif	// INC_RVC_GETOBJFILTER_H
