/******************************************************************************
 *
 * \file sml/doccodes.h
 * \brief SML DOCCODES -- classes used to generate SML documentation
 *
 * \if NODOC
 * $Log: doccodes.h_v $
 * Revision 1.9  2012/12/17 15:42:05  bcolombe
 * Add GetClassExample
 *
 * Revision 1.8  2012/05/21 18:39:42  bcolombe
 * Add UpdateDocs
 *
 * Revision 1.7  2008/10/31 20:28:09  scowan
 * Updated to use non-legacy status classes.
 *
 * Revision 1.6  2007/07/27 14:44:11  dwilliss
 * *** empty log message ***
 *
 * Revision 1.5  2007/07/26 13:51:01  dwilliss
 * added a method
 *
 * Revision 1.4  2007/07/25 20:30:45  dwilliss
 * *** empty log message ***
 *
 * Revision 1.3  2007/05/18 19:46:42  dwilliss
 * added usage parameters
 *
 * Revision 1.2  2007/05/08 15:45:42  dwilliss
 * SML namespace
 *
 * Revision 1.1  2007/05/04 20:46:44  dwilliss
 * Initial revision
 *
 * \endif
*/
/*****************************************************************************/

#ifndef INC_SML_DOCCODES_H
#define INC_SML_DOCCODES_H

#ifndef INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif

#ifndef INC_SML_USAGE_H
#include <sml/usage.h>
#endif

class XMLNODE;
class MISTATUSCONTEXT;

namespace SML {
class SYMBOL;
class CONTEXT;
class FUNC;
class CLASS;

/***************************************************************************/
/**
*		class SML::DOCCODES
*
*		This class can be passed to SML::GetFuncDoc() so that the output
*		string is plain text.  All the methods return the string which
*		should be inserted into the output to switch to the mode indicated
*		by the name (for plain text, they all return empty strings)
*
*		There are derived classes for outputting ANSI or HTML. 
*/
class DOCCODES {
public:

	enum CLASS {
		CLASS_Normal = 0,
		CLASS_FuncGroupDiv,
		CLASS_ItemNameDiv,
		CLASS_Prototype,
		CLASS_ShortDesc,
		CLASS_BoldDiv,
		CLASS_Warning,
		CLASS_Inheritance
		};

	DOCCODES (
		) :
		m_IndentLevel(0)
		{
		}

	virtual ~DOCCODES() {};

// Thanks, Microsoft!
#undef StartDoc
#undef EndDoc
	virtual void Clear() = 0;
	virtual void StartDoc(const char* /*title*/) {}
	virtual void EndDoc() {}
	virtual void StartBold() {};
	virtual void EndBold() {};
	virtual void StartItalic() {};
	virtual void EndItalic() {};
	virtual void StartLine();
	virtual void EndLine() { Append("\n"); }
	virtual void BlankLine() { Append("\n"); }
	virtual void StartIndent() { ++m_IndentLevel; }
	virtual void EndIndent() {if (m_IndentLevel) --m_IndentLevel; }
	virtual void StartCodeFont() {};
	virtual void EndCodeFont() {};
	virtual void InsertHRule() {}

	virtual void StartMainHead() { StartTitle(); }
	virtual void EndMainHead() { EndTitle(); }
	virtual void StartSectionHead() { StartBold(); }
	virtual void EndSectionHead() { EndBold(); }
	virtual void StartNormalFont() {  }
	virtual void EndNormalFont() {  }
	virtual void StartCodeBlock() { StartCodeFont(); }	//! For HTML, use \<pre\>
	virtual void EndCodeBlock() { EndCodeFont(); }
	virtual void StartKeyword() {  SetTextColorBlue(); StartCodeFont(); }
	virtual void EndKeyword() {  EndCodeFont(); SetTextColorNormal(); }
	virtual void StartWarning() {  StartItalic(); SetTextColorRed(); }
	virtual void EndWarning() {  SetTextColorNormal(); EndItalic(); }

	virtual bool IsHTML() { return false; }

	virtual void Append(const char*);	//! Default converts to UC
	virtual void Append(const MIUNICODE*) = 0;
	virtual void StartDiv(CLASS) {}
	virtual void EndDiv() {}
	virtual void StartPrototypeDiv() {}
	virtual void EndPrototypeDiv() {}
	virtual void AppendHyperLink(const char* /*href*/, const MIUNICODE* text) { Append(text);}
	virtual void AppendHyperAnchor(const char* /*name*/) {}
	void AppendML(const MISTRING& str, bool bIndent = false);
	void AppendMsg(TEXTID);				//! From messages.txt [sml] section
	void AppendRes(const char *str);	//! From tntxres.txt

	DOCCODES& operator+=(const char* text) {
		Append(text);
		return *this;
		}

	DOCCODES& operator+=(const MIUNICODE* text) {
		Append(text);
		return *this;
		}

	virtual void SetTextColorRed() {}
	virtual void SetTextColorGreen() {}
	virtual void SetTextColorBlue() {}
	virtual void SetTextColorNormal() {}
	virtual void StartTitle() { StartBold(); }
	virtual void EndTitle() { StartBold(); }

private:
	#ifndef GENERATING_DOXYGEN_OUTPUT
	int m_IndentLevel;
	#endif // GENERATING_DOXYGEN_OUTPUT
	};


//! An intermediate class that SML::DOCCODES_ANSI and SML::DOCCODES_HTML are based
//! on.  Eventually there will be an SML::DOCCODES_RTF which is NOT derived from
//! this one, but only in the MFC world.
class DOCCODES_MISTRING : public DOCCODES {
	public:
		DOCCODES_MISTRING();
		virtual ~DOCCODES_MISTRING() {}
		virtual void Clear() { m_str.Clear(); }
		virtual void Append(const MIUNICODE* str) { m_str += str; }
		virtual void Append(const char* str) { m_str += str; }
		const MISTRING& GetStr() const { return m_str; }
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MISTRING m_str;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

/***************************************************************************/
//! Pass to SML::GetFuncDoc() to get plain text with ANSI codes for bold, color, etc
class DOCCODES_ANSI : public DOCCODES_MISTRING {
public:
	DOCCODES_ANSI() {}
	virtual ~DOCCODES_ANSI() {}
	virtual void StartTitle();
	virtual void EndTitle();
	virtual void StartBold();
	virtual void EndBold();
	virtual void StartItalic();
	virtual void EndItalic();
	virtual void SetTextColorRed();
	virtual void SetTextColorGreen();
	virtual void SetTextColorBlue();
	virtual void SetTextColorNormal();
	virtual void InsertHRule();
	virtual void AppendHyperLink(const char* href, const MIUNICODE* text);
	virtual void AppendHyperAnchor(const char* name);
	};


//! Pass to SML::GetFuncDoc() to get HTML text
class DOCCODES_HTML : public DOCCODES_MISTRING {
public:
	DOCCODES_HTML() {}
	virtual ~DOCCODES_HTML() {}
	virtual void StartDoc(const char* title);
	virtual void EndDoc();
	virtual void StartTitle();
	virtual void EndTitle();
	virtual void StartBold();
	virtual void EndBold();
	virtual void StartItalic();
	virtual void EndItalic();
	virtual void SetTextColorRed();
	virtual void SetTextColorGreen();
	virtual void SetTextColorBlue();
	virtual void SetTextColorNormal();
	virtual void StartLine();
	virtual void EndLine();
	virtual void BlankLine();
	virtual void StartIndent();
	virtual void EndIndent();
	virtual void StartCodeFont();
	virtual void EndCodeFont();
	virtual void StartCodeBlock();
	virtual void EndCodeBlock();
	virtual void InsertHRule();
	virtual void StartDiv(CLASS);
	virtual void EndDiv();
	virtual void StartPrototypeDiv();
	virtual void EndPrototypeDiv();
	virtual void StartMainHead();
	virtual void EndMainHead();
	virtual void StartWarning();
	virtual void EndWarning();
	virtual bool IsHTML() { return true; }
	virtual void AppendHyperLink(const char* href, const MIUNICODE* text);
	virtual void AppendHyperAnchor(const char* name);
	virtual void StartSectionHead();
	virtual void EndSectionHead();
	};

/***************************************************************************/

void BackgroundReadSMLDOCS();

XMLNODE* GetSMLDOCS(bool bInBackground = false);

//! Retrieve the documentation for an SML function.
//!
//! By passing a class derived from SML::DOCCODES, you can
//! generate documentation in plaintext, ANSI, or HTML
//!
//! This function is not a method of SML::CONTEXT because it's
//! not in rvcsys.dll
//!
int GetFuncDoc (
	CONTEXT* context,			//!< Context to work from
	USAGE Usage,
	const SYMBOL *sym,				//!< Function to document
	DOCCODES& codes,
	bool bShortForm = false		//!< Omit example if true
	);

//! Get the example code for a function.
//!
//! This function is not a method of SML::CONTEXT because it's
//! not in rvcsys.dll
int GetFuncExample (
	CONTEXT* context,			//!< Context to work from
	const SYMBOL *sym,			//!< Function to document
	MISTRING& mistr				//!< String to fill in with example
	);

//! Get the description for a function.
//!
//! This function is not a method of SML::CONTEXT because it's
//! not in rvcsys.dll
bool GetFuncDesc (
	CONTEXT* context,			//!< Context to work from
	const FUNC *smlfunc,		//!< Function to document
	MISTRING& mistr				//!< String to fill in with description
	);

//! Get the example code for a class.
//!
//! This function is not a method of SML::CONTEXT because it's
//! not in rvcsys.dll
bool GetClassExample (
	CONTEXT* context,			//!< Context to work from
	const MISTRING& ClassName,
	MISTRING& mistr				//!< String to fill in with example
	);

//! Get the description for a class.
//!
//! This function is not a method of SML::CONTEXT because it's
//! not in rvcsys.dll
bool GetClassDesc (
	CONTEXT* context,			//!< Context to work from
	const MISTRING& ClassName,
	MISTRING& mistr				//!< String to fill in with description
	);

//! Retrieve the documentation for an SML class.
//!
//! By passing a class derived from SML::DOCCODES, you can
//! generate documentation in plaintext, ANSI, or HTML
//!
//! This function is not a method of SML::CONTEXT because it's
//! not in rvcsys.dll
//!
int GetClassDoc (
	CONTEXT* context,			//!< Context to work from
	const CLASS *Class,				//!< Function to document
	DOCCODES& codes,
	bool bInsert = false,
	USAGE usage = USAGE_Undefined	//! Usage of the current context (to determine class availability)
	);


//! Get the documentation for a given SML keyword.
int GetKeywordDoc (
	const char* keyword,
	DOCCODES& codes
	);

//! Get the documentation for a given SML keyword.
bool GetKeywordDesc (
	const char* keyword,
	MISTRING& desc
	);

//! Get the documentation for a given SML keyword.
int GetExampleDoc (
	const MISTRING& name,
	DOCCODES& codes
	);

//! Get the documentation for a given SML keyword.
int GetExampleDoc (
	XMLNODE* node,
	DOCCODES& codes
	);

//! Get the documentation for a given SML keyword.
int GetOperatorDoc (
	const char* op,
	DOCCODES& codes
	);

//! Get the documentation for a given SML operator.
bool GetOperatorDesc (
	const char* op,
	MISTRING& desc
	);

//! Returns an array of keywords for which we have documentation.
//! The array is terminated by a NULL pointer
const char* const* GetKeywordList();
//!
//! Returns an array of operators for which we have documentation.
//! The array is terminated by a NULL pointer
const char* const* GetOperatorList();

int SearchDocs (
	CONTEXT* context,
	const MISTRING& SearchStr,
	DOCCODES& doc,
	MISTATUSCONTEXT& StatusContext
	);

ERRVALUE UpdateDocs (
	const FILEPATH& docspath,
	SML::CONTEXT *context
	);

} // end SML namespace

#define BackgroundReadSMLDOCS	SML::BackgroundReadSMLDOCS
#define SML_GetSMLDOCS SML::GetSMLDOCS
#define SML_GetFuncDoc SML::GetFuncDoc
#define SML_GetFuncExample SML::GetFuncExample
#define SML_GetFuncDesc SML::GetFuncDesc
#define SML_GetClassDesc SML::GetClassDesc
#define SML_GetClassDoc SML::GetClassDoc
#define SML_GetKeywordDoc SML::GetKeywordDoc
#define SML_GetKeywordDesc SML::GetKeywordDesc
#define SML_GetExampleDoc SML::GetExampleDoc
#define SML_GetExampleDoc SML::GetExampleDoc
#define SML_GetOperatorDoc SML::GetOperatorDoc
#define SML_GetOperatorDesc SML::GetOperatorDesc
#define SML_GetKeywordList SML::GetKeywordList
#define SML_GetOperatorList SML::GetOperatorList

#endif		//  INC_SML_DOCCODES_H


