/**
 * \file imagefiltermargin.h <spatmod/imagefiltermargin.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_MARGIN definition.
 *
 * \if NODOC
 * $Id: imagefiltermargin.h_v 1.4 2013/05/20 15:11:30 mju Exp $
 *
 * $Log: imagefiltermargin.h_v $
 * Revision 1.4  2013/05/20 15:11:30  mju
 * Add srcBaseRow/Column to handle subset.
 *
 * Revision 1.3  2006/11/29 22:40:21  mju
 * Add useOverlapBuffers member.
 *
 * Revision 1.2  2006/02/14 18:29:14  mju
 * Add testIterator for sequence validation.
 *
 * Revision 1.1  2005/08/16 15:09:48  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERMARGIN_H
#define	INC_SPATMOD_IMAGEFILTERMARGIN_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_SPATMOD_IMAGEBUFFER_H
#include <spatmod/imagebuffer.h>
#endif

#ifndef	INC_SPATMOD_IMAGESECTION_H
#include <spatmod/imagesection.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to automatically buffer "margins" when iterating.
//! This is useful in implementing "kernel-based" spatial filters, such as low-pass, high-pass, etc.
//! When reading, the buffer must be a window into a larger buffer to allow space for the defined margins.
//! Note that if there is a margin added after the last row/column then the first and last tile sizes will
//! be adjusted accordingly, resulting in a virtual shift in the tile boundaries.  This may have implications
//! when processing concurrently with other images, particularly if any images do not support efficient random
//! access.  In addition, the sequence for iterating must exactly match that specified in the call to PullBegin.
class FILTER_MARGIN : public FILTER {
	public:

		//! Construct with specified margins.
		FILTER_MARGIN (
			STAGE *pSourceImage,						//!< Source image stage
			bool TakeOwnership,						//!< Filter takes ownership of source stage and will delete when needed
			UINT32 RowsBefore,						//!< Margin to add before first row in buffer
			UINT32 ColumnsBefore,					//!< Margin to add before first column in buffer
			UINT32 RowsAfter,							//!< Margin to add after last row in buffer
			UINT32 ColumnsAfter						//!< Margin to add after last column in buffer
			);

		// Destructor.
		virtual ~FILTER_MARGIN ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		INT32 m_RowsBefore;
		INT32 m_ColumnsBefore;
		INT32 m_RowsAfter;
		INT32 m_ColumnsAfter;
		INT32 m_RowsOverlap;
		INT32 m_ColumnsOverlap;
		INT32 m_SrcBaseRow;
		INT32 m_SrcBaseColumn;
		BUFFER m_BufRow;
		BUFFER m_BufRows[2];
		BUFFER m_BufColumn;
		BUFFER m_BufColumns[2];
		int m_LastBufIdxRow;
		int m_NextBufIdxRow;
		int m_LastBufIdxColumn;
		int m_NextBufIdxColumn;
		bool m_UseOverlapBuffers;
		SECTION_ITERATOR m_TestIterator;

		// STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullEnd ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		FILTER_MARGIN (const FILTER_MARGIN&);
		FILTER_MARGIN& operator= (const FILTER_MARGIN&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERMARGIN_H

