/**
 * \file imagesourcegdal.h <spatmod/imagesourcegdal.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::SOURCE_GDAL definition.
 *
 * \if NODOC
 * $Id: imagesourcegdal.h_v 1.4 2013/03/27 17:17:10 bcolombe Exp $
 *
 * $Log: imagesourcegdal.h_v $
 * Revision 1.4  2013/03/27 17:17:10  bcolombe
 * Don't include gdal headers here
 *
 * Revision 1.3  2011/11/14 21:52:49  mju
 * Add construction using memory buffer.
 *
 * Revision 1.2  2010/06/03 16:19:29  bcolombe
 * Added enum DRIVER
 *
 * Revision 1.1  2010/06/03 15:07:46  bcolombe
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESOURCEGDAL_H
#define	INC_SPATMOD_IMAGESOURCEGDAL_H

#ifndef	INC_SPATMOD_IMAGESOURCEFILE_H
#include <spatmod/imagesourcefile.h>
#endif

#ifndef	INC_MI32_FILEPATH_H
#include <mi32/filepath.h>
#endif

#ifndef INC_SPATMOD_IMAGEHELPERRVC_H
#include <spatmod/imagehelperrvc.h>
#endif

class GDALDataset;

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Pipeline source for reading GDAL images from memory or file.
class SOURCE_GDAL : public SOURCE_FILE {
	public:

		enum DRIVER {
			DRIVER_ESRIArcGrid,
			DRIVER_ENVISAT,
			DRIVER_ERDASImagine,
			DRIVER_GIF,
			DRIVER_GMTNetCDF,
			DRIVER_NetCDF,
			DRIVER_NITF,
			DRIVER_PCI
			};

		//! Construct with FILEPATH and optional DriverName.
		explicit SOURCE_GDAL (
			const FILEPATH& FilePath,		//!< Filepath of source
			const char *DriverName = 0		//!< Name of GDALDriver to use, 0 to automatically find correct driver
			);

		//! Construct with FILEPATH and Driver enum.
		//! Uses GDALDriver specified.
		SOURCE_GDAL (
			const FILEPATH& FilePath,		//!< Filepath of source
			DRIVER Driver						//!< Driver to use
			);
		
		//! Construct with memory buffer and optional DriverName.
		explicit SOURCE_GDAL (
			SIMPLE_ARRAY<UINT8>& MemBuffer,	//!< Memory buffer, will be returned empty
			const char *DriverName = 0			//!< Name of GDALDriver to use, 0 to automatically find correct driver
			);

		//! Construct with memory buffer and Driver enum.
		//! Uses GDALDriver specified.
		SOURCE_GDAL (
			SIMPLE_ARRAY<UINT8>& MemBuffer,	//!< Memory buffer, will be returned empty
			DRIVER Driver							//!< Driver to use
			);
		
		//! Destructor.
		virtual ~SOURCE_GDAL ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		GDALDataset *m_pDataset;
		const char *m_DriverName;
		bool m_DoingPull;

		//! Get the celltype of a dataset if contains 24-bit RGB or BGR, or 
		//! 32-bit RGBA, ARGB, CMYB, or BCMY.
		RVC::IMAGE::CELLTYPE GetColorCompCellType (
			GDALDataset *pDataset
			) const;

		void SetDriver (DRIVER driver);

		// BASE overrides.
		virtual ERRVALUE v_GetDimensions (DIMENSIONS& dimensions, double zoom) const;
		virtual ZOOMABILITY v_GetZoomAbility (SIMPLE_ARRAY<double>& ZoomsAllowed) const;
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// STAGE overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullEnd ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		SOURCE_GDAL (const SOURCE_GDAL&);
		SOURCE_GDAL& operator= (const SOURCE_GDAL&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGESOURCEGDAL_H

